from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class DiagnoseCode(BoomParameter):
    """
    Klasse voor boomparameter diagnosecode.

    Boomparameternummer: 230.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, diagnose_attribuut_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat een subtraject voldoet aan een een
        (specialismecode, diagnosecode) combinatie.

        :param str diagnose_attribuut_code: De attribuutcode voor (specialismecode, diagnosecode) combinatie.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_diagnose_attribuut(diagnose_attribuut_code)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.BeslisRegel import BeslisRegel
from kerapu.lbz.Subtraject import Subtraject


class ZorgProductGroepVersie:
    """
    Klasse voor versies zorgproductgroepversies.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self,
                 zorg_product_groep_code: str,
                 beslis_regel: BeslisRegel,
                 begin_datum: str,
                 eind_datum: str):
        """
        Object constructor.

        :param str zorg_product_groep_code: De zorgproductgroepcode.
        :param BeslisRegel beslis_regel: De beslisregel behoorden bij de zorgproductgroepcode.
        :param str begin_datum: Begindatum van het interval waarvoor deze versie van toepassing is.
        :param str eind_datum: Einddatum van het interval waarvoor deze versie van toepassing is.
        """
        self.__zorg_product_groep_code: str = zorg_product_groep_code
        """
        De zorgproductgroepcode.
        """

        self.__beslis_regel: BeslisRegel = beslis_regel
        """
        De beslisregel behorende bij de zorgproductgroepcode.
        """

        self.__begin_datum: str = begin_datum
        """
        Begindatum van het interval waarvoor deze versie van toepassing is.
        """

        self.__eind_datum: str = eind_datum
        """
        Einddatum van het interval waarvoor deze versie van toepassing is.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def is_actueel(self, subtraject: Subtraject) -> bool:
        """
        Geeft True als deze versie actueel is voor een subtraject, anders False.

        :param Subtraject subtraject: Het subtraject.

        :rtype: bool
        """
        return self.__begin_datum <= subtraject.begin_datum <= self.__eind_datum

    # ------------------------------------------------------------------------------------------------------------------
    def klim(self, subtraject: Subtraject) -> str:
        """
        Klimt door de beslisboom een geeft het uiteindelijk gevonden label terug.

        :param Subtraject subtraject: Het subtraject.

        :rtype: str
        """
        return self.__beslis_regel.klim(subtraject)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.attribuut.Attribuut import Attribuut
from kerapu.lbz.Subtraject import Subtraject


class Attribuut21(Attribuut):
    """
    Klasse voor attributen met toetswijze 2 (tussen) en waarde type 1 (numeriek).
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self,
                 attribuut_id: int,
                 boom_parameter_nummer: int,
                 onder_filter_waarde: int,
                 boven_filter_waarde: int) -> None:
        """
        Object constructor.

        :param int attribuut_id: Het ID van dit attribuut.
        :param int boom_parameter_nummer: Het ID van de boomparameter van dit attribuut.
        :param int onder_filter_waarde: De ondergrens.
        :param int boven_filter_waarde: De bovengrens.
        """
        Attribuut.__init__(self, attribuut_id, boom_parameter_nummer)

        self._onder_filter_waarde: int = onder_filter_waarde
        """
        De ondergrens om dit attribuut te laten vuren.
        """

        self._boven_filter_waarde: int = boven_filter_waarde
        """
        De bovengrens om dit attribuut te laten vuren.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen dat de boomparameter voldoet aan de voorwaarde van dit attribuut.

        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        aantal = self._boom_parameter.tel(None, subtraject)

        if self._onder_filter_waarde <= aantal <= self._boven_filter_waarde:
            return 1

        return 0

# ----------------------------------------------------------------------------------------------------------------------



import csv
from typing import Optional, Dict, List, Tuple

from kerapu import clean_code, LEN_SPECIALISME_CODE, LEN_ZORG_TYPE_CODE, clean_str, clean_date


class ZorgType:
    """
    Klasse voor zorgtypen.
    """
    # ------------------------------------------------------------------------------------------------------------------
    __zorg_type_tabel: Dict[Tuple[str, str], List[Dict[str, str]]] = {}
    """
    De zorgtypen referentietabel.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, specialisme_code: str, zorg_type_code: str):
        """
        Object constructor.

        :param str specialisme_code: De code van het uitvoerend specialisme.
        :param str zorg_type_code: De code van deze zorgtype.
        """
        self.__specialisme_code: str = clean_code(specialisme_code, LEN_SPECIALISME_CODE)
        """
        De code van het uitvoerend specialisme.
        """

        self.__zorg_type_code: str = clean_code(zorg_type_code, LEN_ZORG_TYPE_CODE)
        """
        De code van deze zorgtype.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def init_static(folder: str) -> None:
        """
        Initialiseert alle statistische data.

        :param str folder: De folder met alle goupertabellen.
        """
        ZorgType.__lees_zorg_type_tabel(folder)

    # ------------------------------------------------------------------------------------------------------------------
    def __get_zorg_type_referentie(self, datum: str) -> Optional[dict]:
        """
        Zoekt de referentie data voor deze zorg_type in de zorgtype referentietabel.

        :param datum: De begindatum van het subtraject.

        :rtype: dict[str,str]
        """

        if (self.__specialisme_code, self.__zorg_type_code) in self.__zorg_type_tabel:
            for referentie in self.__zorg_type_tabel[(self.__specialisme_code, self.__zorg_type_code)]:
                if referentie['begin_datum'] <= datum <= referentie['eind_datum']:
                    # Een geldige referentie rij gevonden.
                    return referentie

            # Er is geen geldige referentie rij gevonden.
            return None

        else:
            return None

    # ------------------------------------------------------------------------------------------------------------------
    def zorg_type_attribute_aantal(self, zorg_type_attribute_code: str, datum: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) data deze diagnose voldoet aan een (specialismecode, zorgtypecode)
        combinatie op een peildatum.

        :param str zorg_type_attribute_code: De attribuutcode voor (specialismecode, diagnosecode) combinatie.
        :param str datum: De peildatum.

        :rtype: int
        """
        referentie = self.__get_zorg_type_referentie(datum)

        if not referentie:
            # De diagnose komt niet voor in de referentie tabel. Geef 0 terug.
            return 0

        if referentie['zorg_type_attribuut_code'] == zorg_type_attribute_code:
            return 1

        return 0

    # ------------------------------------------------------------------------------------------------------------------
    def zorg_type_cluster_aantal(self, cluster_code: str, cluster_nummer: int, datum: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat deze zorgtype voorkomt in een zorgtypecluster op een peildatum.

        :param str cluster_code: De zorgtypeclustercode.
        :param int cluster_nummer: Het clusternummer (0..2).
        :param str datum: De peildatum.

        :rtype: int
        """
        referentie = self.__get_zorg_type_referentie(datum)

        if not referentie:
            # Deze zorgtype komt niet voor in de referentie tabel. Geef 0 terug.
            return 0

        if cluster_nummer == 0:
            return 1

        if 1 <= cluster_nummer <= 2:
            if referentie['zorg_type_cluster%d' % cluster_nummer] == cluster_code:
                # Deze zorgtype komt voor in het zorgtype cluster.
                return 1

            return 0

        raise RuntimeError("Onbekend clusternummer %d." % cluster_nummer)

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __lees_zorg_type_tabel(folder: str) -> None:
        """
        Leest de zorg_type referentietabel (opgeslagen in CSV).

        :param str folder: De folder met alle goupertabellen.
        """
        with open(folder + '/ZorgTypen.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                specialisme_code = clean_code(regel[0], LEN_SPECIALISME_CODE)
                zorg_type_code = clean_code(regel[1], LEN_ZORG_TYPE_CODE)
                zorg_type_attribuut_code = clean_str(regel[3])
                zorg_type_cluster01 = clean_str(regel[4])
                zorg_type_cluster02 = clean_str(regel[5])
                begin_datum = clean_date(regel[6])
                eind_datum = clean_date(regel[7])

                sleutel = (specialisme_code, zorg_type_code)

                rij = {'specialisme_code':         specialisme_code,
                       'zorg_type_code':           zorg_type_code,
                       'zorg_type_attribuut_code': zorg_type_attribuut_code,
                       'zorg_type_cluster1':       zorg_type_cluster01,
                       'zorg_type_cluster2':       zorg_type_cluster02,
                       'begin_datum':              begin_datum,
                       'eind_datum':               eind_datum}

                if sleutel not in ZorgType.__zorg_type_tabel:
                    ZorgType.__zorg_type_tabel[sleutel] = []

                ZorgType.__zorg_type_tabel[sleutel].append(rij)

        print("Aantal zorgtypen: %d" % (regel_nummer - 1))

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class DiagnoseCluster(BoomParameter):
    """
    Klasse voor boomparameter diagnosecluster.

    Boomparameternummers: 232, 233, 234, 235, 236, 237.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, cluster_nummer: int):
        """
        Object constructor.

        :param int cluster_nummer: Het clusternummer (1..6).
        """
        self._cluster_nummer: int = cluster_nummer
        """
        Het clusternummer (1..6).
        """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, cluster_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat een subtraject voldoet aan een diagnoseclustercode.

        :param str cluster_code: De cluster_code waartegen getest moet worden.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_diagnose_cluster(cluster_code, self._cluster_nummer)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.attribuut.Attribuut import Attribuut
from kerapu.boom.attribuut_groep_koppeling.AttribuutGroepKoppeling import AttribuutGroepKoppeling
from kerapu.boom.attribuut_groep_koppeling.AttribuutGroepKoppeling2 import AttribuutGroepKoppeling2


def maak_attribuut_groep_koppeling(attribute_groep_id: int,
                                   attribuut: Attribuut,
                                   attribuut_toets_wijze: int,
                                   onder_toets_waarde: int,
                                   boven_toets_waarde: int) -> AttribuutGroepKoppeling:
    """
    Fabriek voor het maken van attribuutgroepkoppelingen.

    :param int attribute_groep_id: Het ID van de koppeling.
    :param Attribuut attribuut: Het attribuut van de koppeling.
    :param int attribuut_toets_wijze: De attribuuttoetswijze.
    :param int onder_toets_waarde: De ondergrens.
    :param int boven_toets_waarde: De bovengrens.

    :rtype: AttribuutGroepKoppeling
    """
    if attribuut_toets_wijze == 1:
        # Attribuuttoetswijze 1 wordt thans niet gebruikt in de grouper.
        raise NotImplementedError("Attribuuttoetswijze %d is niet geïmplementeerd." % attribuut_toets_wijze)

    if attribuut_toets_wijze == 2:
        return AttribuutGroepKoppeling2(attribute_groep_id, attribuut, onder_toets_waarde, boven_toets_waarde)

    raise RuntimeError("Onbekende attribuuttoetswijze %d." % attribuut_toets_wijze)

# ----------------------------------------------------------------------------------------------------------------------



from cleo import Command

from kerapu.shredder.BoomBestandenShredder import BoomBestandenShredder
from kerapu.shredder.ReferentieShredder import ReferentieShredder
from kerapu.style.KerapuStyle import KerapuStyle


class ShredderCommand(Command):
    """
    Converteert XML-bestanden met groupertabellen naar CSV-bestanden

    kerapu:shredder
        {XML-bestand : XML-bestand met groupertabellen, b.v. BoomBestanden.xml, Referenties.xml}
        {folder : Folder waar de CSV-bestanden moeten worden opgeslagen}
    """

    # ------------------------------------------------------------------------------------------------------------------
    def handle(self) -> int:
        """
        Executes the command.
        """
        self.output = KerapuStyle(self.input, self.output)

        filename = self.argument('XML-bestand')
        folder = self.argument('folder')

        # Lees de eerste gedeelte van het XML-bestand en bepaal type.
        with open(filename, 'rt', encoding="ascii", errors="surrogateescape") as handle:
            kop = handle.read(1024)

            if '<InlezenBoomBestanden>' in kop:
                shredder = BoomBestandenShredder(self.output, folder)
                shredder.shred_xml_file(filename)

            elif '<InlezenReferenties>' in kop:
                shredder = ReferentieShredder(self.output, folder)
                shredder.shred_xml_file(filename)

            else:
                raise RuntimeError("Tag <InlezenBoomBestanden> of <InlezenReferenties> niet gevonden in '{}'".
                                   format(filename))

        return 0

# ----------------------------------------------------------------------------------------------------------------------



from typing import Union

from cleo import Input, Output
from cleo.styles import CleoStyle


class KerapuStyle(CleoStyle):
    """
    Output style for Kerapu.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, input_object: Input, output_object: Output) -> None:
        """
        Object constructor.

        :param Input input_object: The input object.
        :param Output output_object: The output object.
        """
        CleoStyle.__init__(self, input_object, output_object)

        # Create style notes.
        output_object.get_formatter().add_style('note', 'yellow', None, ['bold'])

        # Create style for file and directory names.
        output_object.get_formatter().add_style('fso', 'white', None, ['bold'])

    # ------------------------------------------------------------------------------------------------------------------
    def text(self, message: Union[str, list, None]) -> None:
        if isinstance(message, list):
            messages = message
        else:
            messages = [message]

        for line in messages:
            self.writeln(' {0}'.format(line))

# ----------------------------------------------------------------------------------------------------------------------



from typing import List

from kerapu.boom.ZorgProductGroepVersie import ZorgProductGroepVersie
from kerapu.lbz.Subtraject import Subtraject


class ZorgProductGroep:
    """
    Klasse voor zorgproductgroepen.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, zorg_product_groep_code: str):
        """
        Object constructor.

        :param str zorg_product_groep_code: De zorgproductgroepcode.
        """
        self._zorg_product_groep_code: str = zorg_product_groep_code
        """
        De zorgproductgroepcode.
        """

        self._versies: List[ZorgProductGroepVersie] = []
        """
        De versies van de zorgproductgroep.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def versie_toevoegen(self, versie: ZorgProductGroepVersie) -> None:
        """
        Voegt een versie toe aan de lijst met versies voor dit zorgproductgroep.

        :param ZorgProductGroepVersie versie: De toe te voegen versie.
        """
        self._versies.append(versie)

    # ------------------------------------------------------------------------------------------------------------------
    def klim(self, subtraject: Subtraject) -> str:
        """
        Bepaalt de zorgproductgroepcode van een subtraject.

        :param subtraject subtraject: Het subtraject.
        """
        versie = self.__get_actuele_versie(subtraject)

        return versie.klim(subtraject)

    # ------------------------------------------------------------------------------------------------------------------
    def __get_actuele_versie(self, subtraject: Subtraject) -> ZorgProductGroepVersie:
        """
        Geeft de actuele versie van dit zorgproductgroep voor een subtraject.

        :param Subtraject subtraject: Het subtraject.

        :rtype: ZorgProductGroepVersie
        """
        for versie in self._versies:
            if versie.is_actueel(subtraject):
                return versie

        raise RuntimeError(
            "Er kan geen actuele versie van zorgproductgroep '%s' gevonden worden voor subtraject '%s" % (
                self._zorg_product_groep_code, subtraject.subtraject_nummer))

# ----------------------------------------------------------------------------------------------------------------------



import csv
import datetime
import os
import random
import shutil
import string
import zipfile
from typing import Iterable, List, Dict

from cleo import Command
from lxml import etree

from kerapu.style.KerapuStyle import KerapuStyle


class TestShredderCommand(Command):
    """
    Converteert XML-bestand met de testset naar een CSV-bestand

    kerapu:test-shredder
        {testset-zip : ZIP-bestand met de testset}
        {testset-csv : Path waar het CSV-bestand met de tests moeten worden opgeslagen}
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __extract_zip_file(self, zip_filename: str, tmp_dir: str):
        """
        Extracts het ZIP-bestand met de testset in een folder.

        :param str zip_filename: Het path naar het ZIP-bestand met de testset.
        :param str tmp_dir: Path naar de folder.
        """
        self.output.writeln('Uitpakken van <fso>{}</fso> in <fso>{}</fso>'.format(zip_filename, tmp_dir))

        with zipfile.ZipFile(zip_filename, 'r') as zip_ref:
            zip_ref.extractall(tmp_dir)

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def ordinal(path: str) -> int:
        """
        Geeft het volgnummer van een test.

        :param str path: Het path naar het XML-bestand met de test case.
        """
        parts = os.path.basename(path).split('_')

        return int(parts[6])

    # ------------------------------------------------------------------------------------------------------------------
    def __lees_test_cases_lijst(self, folder: str) -> List:
        """
        Geeft een lijst met alle bestanden in een folder.

        :param str folder: Het path naar de folder.
        """
        entries = os.listdir(folder)
        filenames = list()
        for entry in entries:
            path = os.path.join(folder, entry)
            if os.path.isfile(path):
                filenames.append(path)

        self.output.writeln('Aantal gevonden test cases: {}'.format(len(filenames)))

        return sorted(filenames, key=TestShredderCommand.ordinal)

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __maak_xpath(parts: Iterable) -> str:
        """
        Maakt een string met een xpath.

        :param tuple parts: The onderdelen van het xpath.

        :rtype: str
        """
        xpath = ''
        for part in parts:
            if xpath:
                xpath += '/'
            xpath += 'xmlns:' + part

        return xpath

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __convert_date(date: str) -> str:
        """
        Converteert een datum in YYYYMMDD formaat naar YYYY-MM-DD format.

        :param str date: De datum in YYYYMMDD format.

        :rtype: str
        """
        return date[:4] + '-' + date[4:6] + '-' + date[6:8]

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __leeftijd_geboorte_datum(date: str, leeftijd: int) -> str:
        """
        Geeft de geboortedatum gegeven een datum en een leeftijd (en de persoon is niet jarig).

        :param str date: De gegeven datum in YYYY-MM-DD format.
        :param int leeftijd: De leeftijd in jaren.

        :rtype: int
        """
        date = datetime.date(int(date[:4]) - leeftijd, int(date[5:7]), int(date[8:10]))
        date -= datetime.timedelta(days=1)

        return date.isoformat()

    # ------------------------------------------------------------------------------------------------------------------
    def __shred_xml_bestand(self, filename: str) -> Dict:
        """
        Leest de relevante data in een XML-bestand met een test case.

        :param str filename: De filenaam van het XML bestand.

        :rtype: dict
        """
        doc = etree.parse(filename)

        xpath = '/soapenv:Envelope/soapenv:Body/xmlns:FICR_IN900101NL04'
        namespaces = {'soapenv': 'http://schemas.xmlsoap.org/soap/envelope/',
                      'xmlns':   'urn:hl7-org:v3'}

        # Lees declaratiecode.
        parts = ('ControlActProcess', 'subject', 'Declaratiedataset', 'component', 'subtraject', 'id')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        declaratie_code = elements[0].get('extension')

        # Lees specialismecode.
        parts = ('ControlActProcess', 'subject', 'Declaratiedataset', 'component', 'subtraject', 'derivedFrom',
                 'zorgtraject', 'responsibleParty', 'assignedPerson', 'code')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        specialisme_code = elements[0].get('code')

        # Lees diagnosecode.
        parts = (
            'ControlActProcess', 'subject', 'Declaratiedataset', 'component', 'subtraject', 'pertinentInformation1',
            'typerendeDiagnose', 'value')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        diagnose_code = elements[0].get('code')

        # Lees zorgtypecode.
        parts = ('ControlActProcess', 'subject', 'Declaratiedataset', 'component', 'subtraject', 'code')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        zorg_type_code = elements[0].get('code') if elements else None

        # Lees zorgvraagcode.
        parts = ('ControlActProcess', 'subject', 'Declaratiedataset', 'component', 'subtraject', 'derivedFrom',
                 'zorgtraject', 'reason', 'zorgvraag', 'value')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        zorg_vraag_code = elements[0].get('code') if elements else None

        # Lees begindatum.
        parts = ('ControlActProcess', 'subject', 'Declaratiedataset', 'component', 'subtraject', 'effectiveTime', 'low')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        begin_datum = self.__convert_date(elements[0].get('value')) if elements else None

        # Lees de geboortedatum van de patient.
        parts = ('ControlActProcess', 'subject', 'Declaratiedataset', 'subject', 'patient', 'subjectOf', 'leeftijd',
                 'value')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        leeftijd = int(elements[0].get('value')) if elements else None
        geboorte_datum = self.__leeftijd_geboorte_datum(begin_datum, leeftijd)

        # Lees het geslacht van de patient.
        parts = ('ControlActProcess', 'subject', 'Declaratiedataset', 'subject', 'patient', 'patientPerson',
                 'administrativeGenderCode')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        geslacht_code = elements[0].get('code') if elements else None

        # Lees de AGB-code van de zorginstelling.
        parts = ('ControlActProcess', 'subject', 'Declaratiedataset', 'author', 'assignedOrganization', 'id')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        zorg_instelling_code = elements[0].get('extension') if elements else None

        # Lees alle zorgactiviteiten.
        zorg_activiteiten = list()
        parts = ('ControlActProcess', 'subject', 'Declaratiedataset', 'component', 'subtraject', 'debit',
                 'zorgactiviteit')
        elements = doc.xpath(xpath + '/' + self.__maak_xpath(parts), namespaces=namespaces)
        for element in elements:
            path = 'xmlns:code'
            sub_elements = element.xpath(path, namespaces=namespaces)
            zorg_activiteit_code = sub_elements[0].get('code') if sub_elements else None

            path = 'xmlns:repeatNumber'
            sub_elements = element.xpath(path, namespaces=namespaces)
            aantal = int(sub_elements[0].get('value')) if sub_elements else None

            zorg_activiteiten.append((zorg_activiteit_code, aantal))

        return {'subtraject_nummer':    os.path.basename(filename),
                'declaratie_code':      declaratie_code,
                'specialisme_code':     specialisme_code,
                'diagnose_code':        diagnose_code,
                'zorg_type_code':       zorg_type_code,
                'zorg_vraag_code':      zorg_vraag_code,
                'begin_datum':          begin_datum,
                'geboorte_datum':       geboorte_datum,
                'geslacht_code':        geslacht_code,
                'zorg_instelling_code': zorg_instelling_code,
                'zorg_activiteiten':    zorg_activiteiten}

    # ----------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __write_subtraject(writer, subtraject: Dict) -> None:
        """
        Schrijft het subtraject met alle zorgactiviteiten naar een CSV-bestand.

        :param writer: De handle naar de CSV writer.
        :param dict subtraject: De details van het subtract.
        """
        writer.writerow((subtraject['subtraject_nummer'],
                         subtraject['specialisme_code'],
                         subtraject['diagnose_code'],
                         subtraject['zorg_type_code'],
                         subtraject['zorg_vraag_code'],
                         subtraject['begin_datum'],
                         subtraject['geboorte_datum'],
                         subtraject['geslacht_code'],
                         subtraject['zorg_instelling_code'],
                         subtraject['declaratie_code']))

        for zorgactiviteit in subtraject['zorg_activiteiten']:
            writer.writerow((zorgactiviteit[0], zorgactiviteit[1]))

    # ----------------------------------------------------------------------------------------------------------------------
    def __extract_files(self, writer, filenames: List) -> None:
        """
        Extract de data van een lijst met XML-bestanden met test cases en schrijft deze data naar een CSV-bestand.

        :param writer:  De handle naar de CSV writer.
        :param list filenames: De lijst met bestandsnamen van XML-bestanden met test cases.
        """
        for filename in filenames:
            subtraject = self.__shred_xml_bestand(filename)
            self.__write_subtraject(writer, subtraject)

    # ------------------------------------------------------------------------------------------------------------------
    def handle(self) -> int:
        """
        Executes the command.
        """
        self.output = KerapuStyle(self.input, self.output)

        zip_filename = self.argument('testset-zip')
        csv_filename = self.argument('testset-csv')
        tmp_dir = '.kerapu-' + ''.join(random.choices(string.ascii_lowercase, k=12))

        os.mkdir(tmp_dir)

        self.__extract_zip_file(zip_filename, tmp_dir)
        files = self.__lees_test_cases_lijst(tmp_dir)

        with open(csv_filename, 'w', encoding='utf-8') as handle:
            csv_writer = csv.writer(handle, dialect=csv.unix_dialect)
            self.__extract_files(csv_writer, files)

        shutil.rmtree(tmp_dir)

        return 0

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class ZorgActiviteitCode(BoomParameter):
    """
    Klasse voor boomparameter zorgactiviteit.

    Boomparameternummers: 300, 400, 500.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, weeg_factor_nummer: int):
        """
        Object constructor.

        :param int weeg_factor_nummer: Weegfactornummer:

                                       * 0: geen weegfactor
                                       * 1: weegfactor 1
                                       * 2: weegfactor 2
        """
        self._weeg_factor_nummer: int = weeg_factor_nummer
        """
        Weegfactornummer.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, zorg_activiteit_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal zorgactiviteiten (met inachtneming van weegfactor) in een subtraject voldoet aan een
        zorgactiviteitcode.

        :param str zorg_activiteit_code: De zorgactiviteitcode.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_zorg_activiteit(zorg_activiteit_code, self._weeg_factor_nummer)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class ZorgVraagCode(BoomParameter):
    """
    Klasse voor boomparameter zorgvraagcode.

    Boomparameternummers: 220.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, zorg_vraag_attribuut_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat de zorgvraag van een subtraject voldoet aan een
        (specialismecode, zorgvraagcode).

        :param str zorg_vraag_attribuut_code: De attribuutcode voor (specialismecode, zorgvraagcode).
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_zorg_vraag_attribuut(zorg_vraag_attribuut_code)

# ----------------------------------------------------------------------------------------------------------------------



from typing import Optional, List

from kerapu import clean_code, LEN_ZORG_ACTIVITEIT_CODE, LEN_ZORG_PRODUCT_GROEP_CODE, LEN_ZORG_PRODUCT_CODE
from kerapu.lbz.Diagnose import Diagnose
from kerapu.lbz.Patient import Patient
from kerapu.lbz.Specialisme import Specialisme
from kerapu.lbz.ZorgActiviteit import ZorgActiviteit
from kerapu.lbz.ZorgInstelling import ZorgInstelling
from kerapu.lbz.ZorgType import ZorgType
from kerapu.lbz.ZorgVraag import ZorgVraag


class Subtraject:
    """
    Klasse voor subtrajecten.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self,
                 subtraject_nummer: str,
                 specialisme_code: str,
                 diagnose_code: str,
                 zorg_type_code: str,
                 zorg_vraag_code: str,
                 begin_datum: str,
                 geboorte_datum: str,
                 geslacht_code: str,
                 zorg_instelling_code: str) -> None:
        """
        Object constructor.

        :param str subtraject_nummer: Het substrajectnummer.
        :param str specialisme_code: Het (uitvoerend)specialismecode.
        :param str diagnose_code:  De Diagnosecode.
        :param str zorg_type_code: Het zorgtypecode.
        :param str zorg_vraag_code: De zorgvraagcode.
        :param str begin_datum:  De begindatum van het subtraject.
        :param str geboorte_datum: De geboortedatum van de patient.
        :param str geslacht_code: De code voor het geslacht van de patient.
        :param str zorg_instelling_code: De AGB-code van de uitvoerende zorginstelling.
        """
        self.__subtraject_nummer: str = subtraject_nummer
        """
        Het subtrajectnummer.
        """

        self.__specialisme: Specialisme = Specialisme(specialisme_code)
        """
        Het uitvoerend specialisme.
        """

        self.__begin_datum: str = begin_datum
        """
        De begindatum van het subtraject.
        """

        self.__patient: Patient = Patient(geboorte_datum, geslacht_code)
        """
        De patient.
        """

        self.__zorg_instelling: ZorgInstelling = ZorgInstelling(zorg_instelling_code)
        """
        De zorginstelling.
        """

        self.__zorg_type: ZorgType = ZorgType(specialisme_code, zorg_type_code)
        """
        Het zorgtype.
        """

        self.__zorg_vraag: ZorgVraag = ZorgVraag(specialisme_code, zorg_vraag_code)
        """
        De zorgvraag.
        """

        self.__diagnose: Diagnose = Diagnose(specialisme_code, diagnose_code)
        """
        De diagnose.
        """

        self.__zorg_activiteiten: List[ZorgActiviteit] = []
        """
        De zorgactiviteiten.
        """

        self.__zorg_product_code: Optional[str] = None
        """
        De zorgproductcode (zoals afgeleid door Kerapu).
        """

        self.__zorg_product_groep_code: Optional[str] = None
        """
        De zorgproductgroepcode (zoals afgeleid door Kerapu).
        """

    # ------------------------------------------------------------------------------------------------------------------
    def add_zorg_activiteit(self, zorg_activiteit_code: str, aantal: int) -> None:
        """
        Voegt een zorgactiviteit toe and dit subtraject.

        :param str zorg_activiteit_code: De zorgactiviteitcode.
        :param int aantal: Het aantal malen (of eenheden) dat de zorgactiviteit is uitgevoerd.
        """
        self.__zorg_activiteiten.append(ZorgActiviteit(zorg_activiteit_code, aantal))

    # ------------------------------------------------------------------------------------------------------------------
    @property
    def begin_datum(self) -> str:
        """
        Geeft de begindatum van dit subtraject.

        :rtype: str
        """
        return self.__begin_datum

    # ------------------------------------------------------------------------------------------------------------------
    @property
    def leeftijd(self) -> int:
        """
        Geeft de leeftijd van de patient van dit subtraject.

        :rtype: int
        """
        return self.__patient.leeftijd(self.__begin_datum)

    # ------------------------------------------------------------------------------------------------------------------
    @property
    def subtraject_nummer(self) -> str:
        """
        Geeft het subtrajectnummer van dit subtraject.

        :rtype: str
        """
        return self.__subtraject_nummer

    # ------------------------------------------------------------------------------------------------------------------
    def telling_behandel_klasse(self, behandel_klasse_code: str, weeg_factor_nummer: int) -> int:
        """
        Geeft het aantal zorgactiviteiten (met inachtneming van weegfactor) dat in dit subtraject voorkomt in een
        behandelklasse.

        :param str behandel_klasse_code: De behandelklassecode waartegen getest moet worden.
        :param int weeg_factor_nummer: Het weegfactornummer (0..2).

        :rtype: int
        """
        aantal = 0
        for zorg_activiteit in self.__zorg_activiteiten:
            aantal += zorg_activiteit.behandel_klasse_aantal(self.__zorg_product_groep_code,
                                                             behandel_klasse_code,
                                                             weeg_factor_nummer,
                                                             self.__begin_datum)

        return aantal

    # ------------------------------------------------------------------------------------------------------------------
    def telling_diagnose_attribuut(self, diagnose_attribuut_code: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat de diagnose van dit subtraject voldoet aan een
        (specialismecode, diagnosecode) combinatie.

        :param str diagnose_attribuut_code: De attribuutcode voor de (specialismecode, diagnosecode) combinatie.

        :rtype: int
        """
        return self.__diagnose.diagnose_attribute_aantal(diagnose_attribuut_code, self.__begin_datum)

    # ------------------------------------------------------------------------------------------------------------------
    def telling_diagnose_cluster(self, cluster_code: str, cluster_nummer: int) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat in dit subtraject voldoet aan een diagnoseclustercode.

        :param str cluster_code: De cluster_code waartegen getest moet worden.
        :param int cluster_nummer: Het clusternummer (1..6).

        :rtype: int
        """
        return self.__diagnose.diagnose_cluster_aantal(cluster_code, cluster_nummer, self.__begin_datum)

    # ------------------------------------------------------------------------------------------------------------------
    def telling_geslacht_code(self, geslacht_code: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat de patient van dit subtraject voldoet aan een geslacht.

        :param str geslacht_code: De geslachtscode waartegen getest moet worden.

        :rtype: int
        """
        if self.__patient.geslacht_code == geslacht_code:
            return 1

        return 0

    # ------------------------------------------------------------------------------------------------------------------
    def telling_specialisme(self, specialisme_code: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat het uitvoerend specialisme van dit subtraject voldoet aan een
        specialismecode.

        :param str specialisme_code: De specialismecode.

        :rtype: int
        """
        return self.__specialisme.specialisme_aantal(specialisme_code, self.__begin_datum)

    # ------------------------------------------------------------------------------------------------------------------
    def telling_specialisme_cluster(self, cluster_code: str, cluster_nummer: int) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat het uitvoerend specialisme van dit subtraject voldoet aan een
        specialismecluster.

        :param str cluster_code: De clustercode waartegen getest moet worden.
        :param int cluster_nummer: Het clusternummer (1..2).

        :rtype: int
        """
        return self.__specialisme.specialisme_cluster_aantal(cluster_code, cluster_nummer, self.__begin_datum)

    # ------------------------------------------------------------------------------------------------------------------
    def telling_zorg_activiteit(self, zorg_activiteit_code: str, weeg_factor_nummer: int) -> int:
        """
        Geeft het aantal zorgactiviteiten (met inachtneming van weegfactor) dat in dit subtraject voldoet aan een
        zorgactiviteitcode.

        :param str zorg_activiteit_code: De zorgactiviteitcode.
        :param int weeg_factor_nummer: Het weegfactornummer (0..2).

        :rtype: int
        """
        aantal = 0
        zorg_activiteit_code = clean_code(zorg_activiteit_code, LEN_ZORG_ACTIVITEIT_CODE)
        for zorg_activiteit in self.__zorg_activiteiten:
            aantal += zorg_activiteit.zorg_activiteit_aantal(zorg_activiteit_code,
                                                             weeg_factor_nummer,
                                                             self.__begin_datum)

        return aantal

    # ------------------------------------------------------------------------------------------------------------------
    def telling_zorg_activiteit_cluster(self,
                                        cluster_code: str,
                                        cluster_nummer: int,
                                        weeg_factor_nummer: int) -> int:
        """
        Geeft het aantal zorgactiviteiten (met inachtneming van weegfactor) dat in dit subtraject voorkomt in een
        zorgactiviteitcluster.

        :param str cluster_code: De zorgactiviteitclustercode.
        :param int cluster_nummer: Het clusternummer (1..10).
        :param int weeg_factor_nummer: Het weegfactornummer (0..2).

        :rtype: int
        """
        aantal = 0
        for zorg_activiteit in self.__zorg_activiteiten:
            aantal += zorg_activiteit.zorg_activiteit_cluster_aantal(cluster_code,
                                                                     cluster_nummer,
                                                                     weeg_factor_nummer,
                                                                     self.__begin_datum)

        return aantal

    # ------------------------------------------------------------------------------------------------------------------
    def telling_zorg_instelling(self, agb_code: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat de zorginstelling van dit subtraject voldoet aan AGB-code.

        :param str agb_code: De AGB-code waaraan de zorginstelling moet voldoen.

        :rtype: int
        """
        return self.__zorg_instelling.zorg_instelling_aantal(agb_code)

    # ------------------------------------------------------------------------------------------------------------------
    def telling_zorg_type_attribuut(self, zorg_type_attribuut_code: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat de zorgtype van dit subtraject voldoet aan een
        (specialismecode, zorgtypecode) combinatie.

        :param str zorg_type_attribuut_code: De attribuutcode voor de (specialismecode, zorgtypecode) combinatie.

        :rtype: int
        """
        return self.__zorg_type.zorg_type_attribute_aantal(zorg_type_attribuut_code, self.__begin_datum)

    # ------------------------------------------------------------------------------------------------------------------
    def telling_zorg_vraag_attribuut(self, zorg_vraag_attribuut_code: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat de zorgvraag van dit subtraject voldoet aan een
        (specialismecode, zorgvraagcode) combinatie.

        :param str zorg_vraag_attribuut_code: De attribuutcode voor de (specialismecode, zorgvraagcode) combinatie.

        :rtype: int
        """
        return self.__zorg_vraag.zorg_vraag_attribute_aantal(zorg_vraag_attribuut_code, self.__begin_datum)

    # ------------------------------------------------------------------------------------------------------------------
    def telling_zorg_vraag_cluster(self, cluster_code: str, cluster_nummer: int) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat de zorgvraag van een subtraject voorkomt in een zorgvraagcluster.

        :param str cluster_code: De cluster_code waartegen getest moet worden.
        :param int cluster_nummer: Het clusternummer (1..2).

        :rtype: int
        """
        return self.__zorg_vraag.zorg_vraag_cluster_aantal(cluster_code, cluster_nummer, self.__begin_datum)

    # ------------------------------------------------------------------------------------------------------------------
    @property
    def zorg_product_code(self) -> Optional[str]:
        """
        Geeft de zorgproductcode van dit subtraject.

        :rtype: str|None
        """
        return self.__zorg_product_code

    # ------------------------------------------------------------------------------------------------------------------
    @zorg_product_code.setter
    def zorg_product_code(self, zorg_product_code: Optional[str]) -> None:
        """
        Zet de zorgproductcode van dit subtraject.

        :param str zorg_product_code: De zorgproductcode.
        """
        if zorg_product_code != '' and zorg_product_code is not None:
            self.__zorg_product_code = clean_code(zorg_product_code, LEN_ZORG_PRODUCT_CODE)
        else:
            self.__zorg_product_groep_code = None

    # ------------------------------------------------------------------------------------------------------------------
    @property
    def zorg_product_groep_code(self) -> Optional[str]:
        """
        Geeft de zorgproductgroepcode van dit subtraject.

        :rtype: str|None
        """
        return self.__zorg_product_groep_code

    # ------------------------------------------------------------------------------------------------------------------
    @zorg_product_groep_code.setter
    def zorg_product_groep_code(self, zorg_product_groep_code: Optional[str]) -> None:
        """
        Zet de zorgproductgroepcode van dit subtraject.

        :param str zorg_product_groep_code: De zorgproductgroepcode.
        """
        if zorg_product_groep_code != '' and zorg_product_groep_code is not None:
            self.__zorg_product_groep_code = clean_code(zorg_product_groep_code, LEN_ZORG_PRODUCT_GROEP_CODE)
        else:
            self.__zorg_product_groep_code = None

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.attribuut.Attribuut import Attribuut
from kerapu.boom.attribuut_groep_koppeling.AttribuutGroepKoppeling import AttribuutGroepKoppeling
from kerapu.lbz.Subtraject import Subtraject


class AttribuutGroepKoppeling2(AttribuutGroepKoppeling):
    """
    Klasse voor attribuutgroepkoppelingen met filtertoetswijze 2 (tussen onder- en bovengrens).
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self,
                 attribute_groep_id: int,
                 attribuut: Attribuut,
                 onder_toets_waarde: int,
                 boven_toets_waarde: int):
        """
        Object constructor.

        :param int attribute_groep_id: Het ID van deze koppeling.
        :param Attribuut attribuut: Het attribuut van deze koppeling.
        :param int onder_toets_waarde: De ondergrens.
        :param int boven_toets_waarde: De bovengrens.
        """
        AttribuutGroepKoppeling.__init__(self, attribute_groep_id, attribuut)

        self._onder_toets_waarde: int = onder_toets_waarde
        """
        De ondergrens om deze koppeling the laten vuren.
        """

        self._boven_toets_waarde: int = boven_toets_waarde
        """
        De bovengrens om deze koppeling the laten vuren.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def test(self, subtraject: Subtraject) -> bool:
        """
        Test of een subtraject voldoet aan een attribuutgroepkoppeling.

        :param Subtraject subtraject: Het subtraject.

        :rtype: bool
        """
        aantal = self._attribuut.tel(subtraject)

        return self._onder_toets_waarde <= aantal <= self._boven_toets_waarde

# ----------------------------------------------------------------------------------------------------------------------



from lxml import etree

from kerapu.shredder.Shredder import Shredder


class ReferentieShredder(Shredder):
    """
    Klasse voor het schreden en opslaan in CSV-formaat van referentietabellen opgeslagen in XML-formaat.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def shred_xml_file(self, filename: str) -> None:
        """
        Slaat de referentietabellen op in CSV-formaat.

        :param str filename: De filenaam van het XML bestand.
        """
        self._io.title('Shredder')

        doc = etree.parse(filename)

        xpath = '/soapenv:Envelope/soapenv:Body/InlezenReferenties/Referenties/'
        namespaces = {'soapenv': 'http://schemas.xmlsoap.org/soap/envelope/'}

        # Extract AfsluitReden.
        table = doc.xpath(xpath + 'AfsluitRedenen/AfsluitReden', namespaces=namespaces)
        fields = ['AfsluitRedenCode',
                  'AfsluitRedenOmschrijving',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'AfsluitRedenen.csv', fields, fields)

        # Extract BehandelKlasse.
        table = doc.xpath(xpath + 'BehandelKlassen/BehandelKlasse', namespaces=namespaces)
        fields = ['ZorgProductGroepCode',
                  'ZorgActiviteitCode',
                  'BehandelKlasseCode',
                  'BehandelKlasseOmschrijving',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'BehandelKlassen.csv', fields, fields)

        # Extract Diagnose.
        table = doc.xpath(xpath + 'Diagnosen/Diagnose', namespaces=namespaces)
        fields = ['SpecialismeCode',
                  'DiagnoseCode',
                  'DiagnoseOmschrijving',
                  'DiagnoseAttribuutCode',
                  'ICD10DiagnoseCode',
                  'DiagnoseCluster1',
                  'DiagnoseCluster2',
                  'DiagnoseCluster3',
                  'DiagnoseCluster4',
                  'DiagnoseCluster5',
                  'DiagnoseCluster6',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        xpaths = ['SpecialismeCode',
                  'DiagnoseCode',
                  'DiagnoseOmschrijving',
                  'DiagnoseAttribuutCode',
                  'ICD10DiagnoseCode',
                  "DiagnoseCluster/DiagnoseClusterItem[@Key='1']",
                  "DiagnoseCluster/DiagnoseClusterItem[@Key='2']",
                  "DiagnoseCluster/DiagnoseClusterItem[@Key='3']",
                  "DiagnoseCluster/DiagnoseClusterItem[@Key='4']",
                  "DiagnoseCluster/DiagnoseClusterItem[@Key='5']",
                  "DiagnoseCluster/DiagnoseClusterItem[@Key='6']",
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'Diagnosen.csv', fields, xpaths)

        # Extract Geslacht.
        table = doc.xpath(xpath + 'Geslachten/Geslacht', namespaces=namespaces)
        fields = ['GeslachtCodeNEN',
                  'GeslachtCodeHL7',
                  'GeslachtOmschrijving',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'Geslachten.csv', fields, fields)

        # Extract LimitatieMachtiging.
        table = doc.xpath(xpath + 'LimitatieMachtigingen/LimitatieMachtiging', namespaces=namespaces)
        fields = ['SpecialismeCode',
                  'DiagnoseCode',
                  'DiagnoseOmschrijving',
                  'ZorgActiviteitCode',
                  'ZorgActiviteitOmschrijving',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'LimitatieMachtigingen.csv', fields, fields)

        # Extract Specialisme.
        table = doc.xpath(xpath + 'Specialismen/Specialisme', namespaces=namespaces)
        fields = ['SpecialismeCode',
                  'SpecialismeOmschrijving',
                  'SpecialismeIndicatie',
                  'SpecialismeOID',
                  'SpecialismeCluster1',
                  'SpecialismeCluster2',
                  'IndicatieAanspraakbeperking',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        xpaths = ['SpecialismeCode',
                  'SpecialismeOmschrijving',
                  'SpecialismeIndicatie',
                  'SpecialismeOID',
                  'SpecialismeCluster/SpecialismeClusterItem[(@Key=1)]',
                  'SpecialismeCluster/SpecialismeClusterItem[(@Key=2)]',
                  'IndicatieAanspraakbeperking',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'Specialismen.csv', fields, xpaths)

        # Extract VertaalZorgActiviteit.
        table = doc.xpath(xpath + 'VertaalZorgActiviteiten/VertaalZorgActiviteit', namespaces=namespaces)
        fields = ['ZorgActiviteitCode',
                  'ZorgActiviteitOmschrijving',
                  'ZorgActiviteitCodeOud',
                  'ZorgActiviteitOmschrijvingOud',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'VertaalZorgActiviteiten.csv', fields, fields)

        # Extract ZorgActiviteit.
        table = doc.xpath(xpath + 'ZorgActiviteiten/ZorgActiviteit', namespaces=namespaces)
        fields = ['ZorgActiviteitCode',
                  'ZorgActiviteitOmschrijving',
                  'ZorgActiviteitCluster01',
                  'ZorgActiviteitCluster02',
                  'ZorgActiviteitCluster03',
                  'ZorgActiviteitCluster04',
                  'ZorgActiviteitCluster05',
                  'ZorgActiviteitCluster06',
                  'ZorgActiviteitCluster07',
                  'ZorgActiviteitCluster08',
                  'ZorgActiviteitCluster09',
                  'ZorgActiviteitCluster10',
                  'ZorgActiviteitWeegFactor1',
                  'ZorgActiviteitWeegFactor2',
                  'WBMVCode',
                  'InnovatieCode',
                  'AanspraakCode',
                  'TariefType',
                  'Zorgprofielklassecode',
                  'OpNota',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        xpaths = ['ZorgActiviteitCode',
                  'ZorgActiviteitOmschrijving',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=1]',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=2]',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=3]',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=4]',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=5]',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=6]',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=7]',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=8]',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=9]',
                  'ZorgActiviteitCluster/ZorgActiviteitClusterItem[@Key=10]',
                  'ZorgActiviteitWeegFactor/ZorgActiviteitWeegFactorItem[@Key=1]',
                  'ZorgActiviteitWeegFactor/ZorgActiviteitWeegFactorItem[@Key=2]',
                  'WBMVCode',
                  'InnovatieCode',
                  'AanspraakCode',
                  'TariefType',
                  'Zorgprofielklassecode',
                  'OpNota',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'ZorgActiviteiten.csv', fields, xpaths)

        # Extract ZorgProductGroep.
        table = doc.xpath(xpath + 'ZorgProductGroepen/ZorgProductGroep', namespaces=namespaces)
        fields = ['ZorgProductGroepCode',
                  'ZorgProductGroepOmschrijving',
                  'BeslisRegelStart',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'ZorgProductGroepen.csv', fields, fields)

        # Extract ZorgType.
        table = doc.xpath(xpath + 'ZorgTypen/ZorgType', namespaces=namespaces)
        fields = ['SpecialismeCode',
                  'ZorgTypeCode',
                  'ZorgTypeOmschrijving',
                  'ZorgTypeAttribuutCode',
                  'ZorgTypeCluster1',
                  'ZorgTypeCluster2',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        xpaths = ['SpecialismeCode',
                  'ZorgTypeCode',
                  'ZorgTypeOmschrijving',
                  'ZorgTypeAttribuutCode',
                  'ZorgTypeCluster/ZorgTypeClusterItem[(@Key=1)]',
                  'ZorgTypeCluster/ZorgTypeClusterItem[(@Key=2)]',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'ZorgTypen.csv', fields, xpaths)

        # Extract ZorgVraag.
        table = doc.xpath(xpath + 'ZorgVragen/ZorgVraag', namespaces=namespaces)
        fields = ['SpecialismeCode',
                  'ZorgVraagCode',
                  'ZorgVraagOmschrijving',
                  'ZorgVraagAttribuutCode',
                  'ZorgVraagCluster1',
                  'ZorgVraagCluster2',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        xpaths = ['SpecialismeCode',
                  'ZorgVraagCode',
                  'ZorgVraagOmschrijving',
                  'ZorgVraagAttribuutCode',
                  'ZorgVraagCluster/ZorgVraagClusterItem[(@Key=1)]',
                  'ZorgVraagCluster/ZorgVraagClusterItem[(@Key=2)]',
                  'BeginDatum',
                  'EindDatum',
                  'VersieDatum']
        self.extract_table(table, 'ZorgVragen.csv', fields, xpaths)

# ----------------------------------------------------------------------------------------------------------------------



import abc

from kerapu.boom.boom_parameter import create_boom_parameter, BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class Attribuut:
    """
    Abstract klasse voor attributen.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, attribuut_id: int, boom_parameter_nummer: int):
        """
        Object constructor.

        :param int attribuut_id: Het ID van dit attribuut.
        :param int boom_parameter_nummer: Het ID van de boomparameter va dit attribuut.
        """
        self._attribuut_id: int = attribuut_id
        """
        Het ID van dit attribuut.
        """

        self._boom_parameter: BoomParameter = create_boom_parameter(boom_parameter_nummer)
        """
        De boomparameter van dit attribuut.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @abc.abstractmethod
    def tel(self, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen dat de boomparameter voldoet aan de voorwaarde van dit attribuut.

        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        pass

# ----------------------------------------------------------------------------------------------------------------------



import abc

from kerapu.lbz.Subtraject import Subtraject


class BoomParameter:
    """
    Abstracte klasse voor boomparameters.
    """

    # ------------------------------------------------------------------------------------------------------------------
    @abc.abstractmethod
    def tel(self, waarde, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen dat de boomparameter voldoet aan een waarde.

        :param [int|str] waarde: De waarde waartegen getest moet worden.
        :param kerapu.lbz.Subtraject.Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        raise Exception("Not implemented")

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.attribuut.Attribuut import Attribuut
from kerapu.lbz.Subtraject import Subtraject


class Attribuut12(Attribuut):
    """
    Klasse voor attributen met toetswijze 1 (gelijk) en waarde type 2 (alfanumeriek).
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, attribuut_id: int, boom_parameter_nummer: int, filter_waarde: str):
        """
        Object constructor.

        :param int attribuut_id: Het ID van dit attribuut.
        :param int boom_parameter_nummer: Het ID van de boomparameter van dit attribuut.
        :param str filter_waarde: De filter waarde.
        """
        Attribuut.__init__(self, attribuut_id, boom_parameter_nummer)

        self._filter_waarde: str = filter_waarde
        """
        De waarde om dit attribuut the laten vuren.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen dat de boomparameter voldoet aan de voorwaarde van dit attribuut.

        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return self._boom_parameter.tel(self._filter_waarde, subtraject)

# ----------------------------------------------------------------------------------------------------------------------



import csv
from typing import Optional, Dict, List, Tuple

from kerapu import clean_code, clean_date, clean_str, LEN_DIAGNOSE_CODE, LEN_SPECIALISME_CODE


class Diagnose:
    """
    Klasse voor diagnosen.
    """
    # ------------------------------------------------------------------------------------------------------------------
    __diagnose_tabel: Dict[Tuple[str, str], List[Dict[str, str]]] = {}
    """
    De diagnosen referentietabel.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, specialisme_code: str, diagnose_code: str):
        """
        Object constructor.

        :param str specialisme_code: De code van het uitvoerend specialisme.
        :param str diagnose_code: De code van deze diagnose.
        """
        self.__specialisme_code: str = clean_code(specialisme_code, LEN_SPECIALISME_CODE)
        """
        De code van het uitvoerend specialisme.
        """

        self.__diagnose_code: str = clean_code(diagnose_code, LEN_DIAGNOSE_CODE)
        """
        De code van deze diagnose.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __lees_diagnose_tabel(folder: str) -> None:
        """
        Leest de diagnose referentietabel (opgeslagen in CSV).

        :param str folder: De folder met alle goupertabellen.
        """
        with open(folder + '/Diagnosen.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                specialisme_code = clean_code(regel[0], LEN_SPECIALISME_CODE)
                diagnose_code = clean_code(regel[1], LEN_DIAGNOSE_CODE)
                diagnose_attribute_code = clean_str(regel[3])
                diagnose_cluster01 = clean_str(regel[5])
                diagnose_cluster02 = clean_str(regel[6])
                diagnose_cluster03 = clean_str(regel[7])
                diagnose_cluster04 = clean_str(regel[8])
                diagnose_cluster05 = clean_str(regel[9])
                diagnose_cluster06 = clean_str(regel[10])
                begin_datum = clean_date(regel[11])
                eind_datum = clean_date(regel[12])

                sleutel = (specialisme_code, diagnose_code)

                rij = {'specialisme_code':        specialisme_code,
                       'diagnose_code':           diagnose_code,
                       'diagnose_attribute_code': diagnose_attribute_code,
                       'diagnose_cluster1':       diagnose_cluster01,
                       'diagnose_cluster2':       diagnose_cluster02,
                       'diagnose_cluster3':       diagnose_cluster03,
                       'diagnose_cluster4':       diagnose_cluster04,
                       'diagnose_cluster5':       diagnose_cluster05,
                       'diagnose_cluster6':       diagnose_cluster06,
                       'begin_datum':             begin_datum,
                       'eind_datum':              eind_datum}

                if sleutel not in Diagnose.__diagnose_tabel:
                    Diagnose.__diagnose_tabel[sleutel] = []

                Diagnose.__diagnose_tabel[sleutel].append(rij)

        print("Aantal diagnosen: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def init_static(folder: str) -> None:
        """
        Initialiseert alle statistische data.

        :param str folder: De folder met alle goupertabellen.
        """
        Diagnose.__lees_diagnose_tabel(folder)

    # ------------------------------------------------------------------------------------------------------------------
    def __diagnose_referentie(self, datum: str) -> Optional[Dict]:
        """
        Zoekt de referentie data voor deze diagnose in de diagnosen referentietabel.

        :param str datum: De begindatum van het subtraject.
        :rtype: dict[str,str]
        """
        if (self.__specialisme_code, self.__diagnose_code) in self.__diagnose_tabel:
            for referentie in self.__diagnose_tabel[(self.__specialisme_code, self.__diagnose_code)]:
                if referentie['begin_datum'] <= datum <= referentie['eind_datum']:
                    # Een geldige referentie rij gevonden.
                    return referentie

            # Er is geen geldige referentie rij gevonden.
            return None

        else:
            return None

    # ------------------------------------------------------------------------------------------------------------------
    def diagnose_attribute_aantal(self, diagnose_attribute_code: str, datum: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) data deze diagnose voldoet aan een (specialismecode, diagnosecode)
        op een peildatum.

        :param str diagnose_attribute_code: De attribuutcode voor de (specialismecode, diagnosecode) combinatie.
        :param str datum: De peildatum.

        :rtype: int
        """
        referentie = self.__diagnose_referentie(datum)

        if not referentie:
            # De diagnose komt niet voor in de referentie tabel. Geef 0 terug.
            return 0

        if referentie['diagnose_attribute_code'] == diagnose_attribute_code:
            return 1

        return 0

    # ------------------------------------------------------------------------------------------------------------------
    def diagnose_cluster_aantal(self, cluster_code: str, cluster_nummer: int, datum: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) data deze diagnose voorkomt in een diagnosecodecluster op een peildatum.

        :param str cluster_code: De diagnoseclustercode.
        :param int cluster_nummer: De clusternummer (0..6).
        :param str datum: De peildatum.

        :rtype: int
        """
        referentie = self.__diagnose_referentie(datum)

        if not referentie:
            # De diagnose komt niet voor in de referentie tabel. Geef 0 terug.
            return 0

        if cluster_nummer == 0:
            return 1

        if 1 <= cluster_nummer <= 6:
            if referentie['diagnose_cluster%d' % cluster_nummer] == cluster_code:
                # Deze diagnose komt voor in het gevraagde cluster.
                return 1

            return 0

        raise RuntimeError("Onbekend clusternummer %d." % cluster_nummer)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class BehandelKlasse(BoomParameter):
    """
    Klasse voor boomparameter behandelklasse: som van aantal (met en zonder weegfactor).

    Boomparameternummers: 351, 451, 551.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, weeg_factor_nummer: int):
        """
        Object contructor.

        :param int weeg_factor_nummer: Weegfactornummer:

                                       * 0: geen weegfactor
                                       * 1: weegfactor 1
                                       * 2: weegfactor 2
        """
        self._weeg_factor_nummer: int = weeg_factor_nummer
        """
        Weegfactornummer
        """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, behandel_klasse_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal zorgactiviteiten (met inachtneming van weegfactor) in een subtraject voorkomt in een
        behandelklasse.

        :param str behandel_klasse_code: De behandelklassecode waartegen getest moet worden.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_behandel_klasse(behandel_klasse_code, self._weeg_factor_nummer)

# ----------------------------------------------------------------------------------------------------------------------



from lxml import etree

from kerapu.shredder.Shredder import Shredder


class BoomBestandenShredder(Shredder):
    """
    Klasse voor het schreden en opslaan in CSV-formaat van boombestanden opgeslagen in XML-formaat.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def shred_xml_file(self, filename: str) -> None:
        """
        Slaat de boombestanden op in CSV-formaat.

        :param str filename: De filenaam van het XML bestand.
        """
        self._io.title('Shredder')

        doc = etree.parse(filename)

        xpath = '/soapenv:Envelope/soapenv:Body/InlezenBoomBestanden/BoomBestanden/'
        namespaces = {'soapenv': 'http://schemas.xmlsoap.org/soap/envelope/'}

        # Extract beslis regels.
        table = doc.xpath(xpath + 'BeslisRegels/BeslisRegel', namespaces=namespaces)
        fields = ['BeslisRegelId',
                  'AttribuutGroepId',
                  'BeslisRegelTrue',
                  'BeslisRegelFalse',
                  'LabelTrue',
                  'LabelFalse',
                  'IndicatieAanspraakbeperking',
                  'VersieDatum']
        self.extract_table(table, 'BeslisRegels.csv', fields, fields)

        # Extract attribuut groep.
        table = doc.xpath(xpath + 'AttribuutGroepen/AttribuutGroep', namespaces=namespaces)
        fields = ['AttribuutGroepId',
                  'AttribuutGroepOmschrijving',
                  'AantalVoorwaardenVoorTrue',
                  'VersieDatum']
        self.extract_table(table, 'AttribuutGroepen.csv', fields, fields)

        # Extract attribuut groep koppeling.
        table = doc.xpath(xpath + 'AttribuutGroepKoppelingen/AttribuutGroepKoppeling', namespaces=namespaces)
        fields = ['AttribuutGroepKoppelingId',
                  'AttribuutGroepId',
                  'AttribuutId',
                  'AttribuutToetsWijze',
                  'OnderToetsWaarde',
                  'BovenToetsWaarde',
                  'VersieDatum']
        self.extract_table(table, 'AttribuutGroepKoppelingen.csv', fields, fields)

        # Extract attributen
        table = doc.xpath(xpath + 'Attributen/Attribuut', namespaces=namespaces)
        fields = ['AttribuutId',
                  'AttribuutOmschrijving',
                  'BoomParameterNummer',
                  'FilterToetsWijze',
                  'FilterWaardeType',
                  'OnderFilterWaarde',
                  'BovenFilterWaarde',
                  'VersieDatum']
        self.extract_table(table, 'Attributen.csv', fields, fields)

        # Extract boomparameters.
        table = doc.xpath(xpath + 'BoomParameters/BoomParameter', namespaces=namespaces)
        fields = ['BoomParameterNummer',
                  'Omschrijving',
                  'TabelNaam',
                  'VeldNaam']
        self.extract_table(table, 'BoomParameters.csv', fields, fields)

# ----------------------------------------------------------------------------------------------------------------------



import csv
from typing import List

from lxml.etree import Element

from kerapu.style.KerapuStyle import KerapuStyle


class Shredder:
    """
    Klasse voor het schreden van XML-bestanden en opslaan in CSV-formaat.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, io: KerapuStyle, target_dir: str):
        """
        Object constructor.

        :param str target_dir: De folder waar de CSV-bestanden moeten worden opgeslagen.
        """
        self._io: KerapuStyle = io
        """
        The output decorator.
        """

        self.__target_dir: str = target_dir
        """
        De folder waar de CSV-bestanden moeten worden opgeslagen.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def extract_field(element: Element, tag: str) -> str:
        """
        Extracts de waarde van een XML element.

        :param Element element: Het parent XML element.
        :param str tag: De tag van het gevraagde XML-element.

        :rtype: str
        """
        elements = element.xpath(tag, namespaces={'soapenv': 'http://schemas.xmlsoap.org/soap/envelope/'})
        if elements:
            return elements[0].text

        return ''

    # ------------------------------------------------------------------------------------------------------------------
    def extract_table(self, table: Element, filename: str, fields: List, xpaths: List) -> None:
        """
        Extracts een groupertabel uit XML een slaat de tabel op in een CSV-bestand.

        :param Element table: De naam van de groupertabel.
        :param str filename: De filenaam van het CSV-bestaand.
        :param list fields: Een lijst met velden (d.w.z. kolomen in het CSV-bestand).
        :param list xpaths: Een lijst met xpath voor het extracten van de bovenstaande velden.
        """
        # Sanity test.
        if len(fields) != len(xpaths):
            raise ValueError("fields and xpaths must have equal length")

        # Open the file and create CSV writer.
        file = open(self.__target_dir + '/' + filename, 'w', encoding='utf-8')
        writer = csv.writer(file)

        # Write header row.
        writer.writerow(fields)

        # Write all rows in the XML 'table'.
        row_count = 0
        for element in table:
            row = []
            for xpath in xpaths:
                row.append(Shredder.extract_field(element, xpath))
            writer.writerow(row)
            row_count += 1

        # Close the file.
        file.close()

        self._io.text('Wrote {:6d} rows to <fso>{}</fso>'.format(row_count, filename))

# ----------------------------------------------------------------------------------------------------------------------



from kerapu import clean_code, LEN_ZORG_INSTELLING_CODE


class ZorgInstelling:
    """
    Klasse voor zorginstellingen.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, zorg_instelling_code: str):
        """
        Object constructor.

        :param str zorg_instelling_code: De code van deze zorginstelling.
        """
        self.__zorg_instelling_code: str = clean_code(zorg_instelling_code, LEN_ZORG_INSTELLING_CODE)
        """
        De code van deze zorginstelling.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def zorg_instelling_aantal(self, zorg_instelling_code: str) -> int:
        """
        Geeft het aantal malen (met inachtneming van weegfactor) dat deze zorginstelling voldoet aan een
        zorginstellingcode.

        :param str zorg_instelling_code: De AGB-code van de zorginstelling.

        :rtype: int
        """
        return 1 if self.__zorg_instelling_code == clean_code(zorg_instelling_code, LEN_ZORG_INSTELLING_CODE) else 0

    # ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.attribuut.Attribuut import Attribuut
from kerapu.boom.attribuut.Attribuut12 import Attribuut12
from kerapu.boom.attribuut.Attribuut21 import Attribuut21


def maak_attribuut(attribuut_id: int,
                   boom_parameter_nummer: int,
                   filter_toets_wijze: int,
                   filter_waarde_type: int,
                   onder_filter_waarde: str,
                   boven_filter_waarde: str) -> Attribuut:
    """
    Fabriek voor het maken van attributen.

    :param int attribuut_id: Het ID van het attribuut.
    :param int boom_parameter_nummer: Het ID van de boomparameter van het attribuut.
    :param int filter_toets_wijze: De filtertoetswijze
    :param int filter_waarde_type: Het type van de waarde van het attribuut.
    :param str onder_filter_waarde: De ondergrens.
    :param str boven_filter_waarde: De bovengrens.

    :rtype: Attribuut
    """
    if filter_toets_wijze == 1 and filter_waarde_type == 2:
        return Attribuut12(attribuut_id, boom_parameter_nummer, onder_filter_waarde)

    if filter_toets_wijze == 2 and filter_waarde_type == 1:
        return Attribuut21(attribuut_id, boom_parameter_nummer, int(onder_filter_waarde), int(boven_filter_waarde))

    if not 1 <= filter_toets_wijze <= 2:
        raise RuntimeError("Onbekende filtertoetswijze %d." % filter_toets_wijze)

    if not 1 <= filter_waarde_type <= 3:
        raise RuntimeError("Onbekende type %d van de waarde van het attribuut." % filter_waarde_type)

    raise NotImplementedError("Attribuut met filtertoetswijze %d en waarde type %d is niet geïmplementeerd" % (
        filter_toets_wijze, filter_waarde_type))

# ----------------------------------------------------------------------------------------------------------------------



from typing import List

from cleo import Application, Command

from kerapu.command.ShredderCommand import ShredderCommand
from kerapu.command.TestsetShredderCommand import TestShredderCommand


class KerapuApplication(Application):
    """
    The Kerapu application.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self):
        """
        Object constructor.
        """
        Application.__init__(self, 'kerapu', '2.0.3')

    # ------------------------------------------------------------------------------------------------------------------
    def get_default_commands(self) -> List[Command]:
        """
        Returns the default commands of this application.

        :rtype: list[Command]
        """
        commands = Application.get_default_commands(self)

        # Kerapu:
        commands.append(ShredderCommand())
        commands.append(TestShredderCommand())

        return commands

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BehandelKlasse import BehandelKlasse
from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.boom.boom_parameter.DiagnoseCluster import DiagnoseCluster
from kerapu.boom.boom_parameter.DiagnoseCode import DiagnoseCode
from kerapu.boom.boom_parameter.Geslacht import Geslacht
from kerapu.boom.boom_parameter.Leeftijd import Leeftijd
from kerapu.boom.boom_parameter.SpecialismeCluster import SpecialismeCluster
from kerapu.boom.boom_parameter.SpecialismeCode import SpecialismeCode
from kerapu.boom.boom_parameter.ZorgActiviteitCluster import ZorgActiviteitCluster
from kerapu.boom.boom_parameter.ZorgActiviteitCode import ZorgActiviteitCode
from kerapu.boom.boom_parameter.ZorgInstellingCode import ZorgInstellingCode
from kerapu.boom.boom_parameter.ZorgTypeCode import ZorgTypeCode
from kerapu.boom.boom_parameter.ZorgVraagCluster import ZorgVraagCluster
from kerapu.boom.boom_parameter.ZorgVraagCode import ZorgVraagCode

# ----------------------------------------------------------------------------------------------------------------------
_boom_parameters = {}
"""
Poel met alle boomparameters
"""


# ----------------------------------------------------------------------------------------------------------------------
def create_boom_parameter(boom_parameter_nummer: int) -> BoomParameter:
    """
    Een fabriek met hergebruik voor het maken van boomparameters. Het aanroepen van deze functie met hetzelfde
    boomparameternummer zal tekens het zelfde object opleveren.

    :param int boom_parameter_nummer: Het nummer van de boomparameter.

    :rtype: BoomParameter
    """
    if boom_parameter_nummer in _boom_parameters:
        return _boom_parameters[boom_parameter_nummer]

    _boom_parameters[boom_parameter_nummer] = _create_boom_parameter(boom_parameter_nummer)

    return _boom_parameters[boom_parameter_nummer]


# ----------------------------------------------------------------------------------------------------------------------
def _create_boom_parameter(boom_parameter_nummer: int) -> BoomParameter:
    """
    Een fabriek voor het maken van boomparameters.

    :param int boom_parameter_nummer: Het nummer van de boomparameter.

    :rtype: BoomParameter
    """
    if boom_parameter_nummer == 100:
        return Leeftijd()

    if boom_parameter_nummer == 101:
        return Geslacht()

    if boom_parameter_nummer == 110:
        return ZorgInstellingCode()

    if boom_parameter_nummer == 111:
        # Zorginstellingscluster 1
        # Deze boomparameter wordt thans niet gebruikt door de grouper.
        raise NotImplementedError('Boomparameter %d is niet geïmplementeerd.' % boom_parameter_nummer)

    if boom_parameter_nummer == 112:
        # Zorginstellingscluster 2
        # Deze boomparameter wordt thans niet gebruikt door de grouper.
        raise NotImplementedError('Boomparameter %d is niet geïmplementeerd.' % boom_parameter_nummer)

    if boom_parameter_nummer == 200:
        return SpecialismeCode()

    if boom_parameter_nummer == 201:
        return SpecialismeCluster(1)

    if boom_parameter_nummer == 202:
        return SpecialismeCluster(2)

    if boom_parameter_nummer == 210:
        return ZorgTypeCode()

    if boom_parameter_nummer == 211:
        # Zorgtypecluster 1
        # Deze boomparameter wordt thans niet gebruikt door de grouper.
        raise NotImplementedError('Boomparameter %d is niet geïmplementeerd.' % boom_parameter_nummer)

    if boom_parameter_nummer == 212:
        # Zorgtypecluster 2
        # Deze boomparameter wordt thans niet gebruikt door de grouper.
        raise NotImplementedError('Boomparameter %d is niet geïmplementeerd.' % boom_parameter_nummer)

    if boom_parameter_nummer == 220:
        return ZorgVraagCode()

    if boom_parameter_nummer == 221:
        return ZorgVraagCluster(1)

    if boom_parameter_nummer == 222:
        return ZorgVraagCluster(2)

    if boom_parameter_nummer == 230:
        return DiagnoseCode()

    if boom_parameter_nummer == 231:
        # ICD-diagnosecode
        # Deze boomparameter wordt thans niet gebruikt door de grouper.
        raise NotImplementedError('Boomparameter %d is niet geïmplementeerd.' % boom_parameter_nummer)

    if 232 <= boom_parameter_nummer <= 237:
        return DiagnoseCluster(boom_parameter_nummer - 231)

    if boom_parameter_nummer == 241:
        # Begindatum subtraject
        # Deze boomparameter wordt thans niet gebruikt door de grouper.
        raise NotImplementedError('Boomparameter %d is niet geïmplementeerd.' % boom_parameter_nummer)

    if boom_parameter_nummer == 300:
        return ZorgActiviteitCode(0)

    if 301 <= boom_parameter_nummer <= 310:
        return ZorgActiviteitCluster(boom_parameter_nummer - 300, 0)

    if boom_parameter_nummer == 351:
        return BehandelKlasse(0)

    if boom_parameter_nummer == 400:
        return ZorgActiviteitCode(1)

    if 401 <= boom_parameter_nummer <= 410:
        return ZorgActiviteitCluster(boom_parameter_nummer - 400, 1)

    if boom_parameter_nummer == 451:
        # Behandelklasse – som van (aantal * weegfactor 1)
        # Deze boomparameter wordt thans niet gebruikt door de grouper.
        # return BehandelKlasse(1)
        raise NotImplementedError('Boomparameter %d is niet geïmplementeerd.' % boom_parameter_nummer)

    if boom_parameter_nummer == 500:
        return ZorgActiviteitCode(2)

    if 501 <= boom_parameter_nummer <= 510:
        return ZorgActiviteitCluster(boom_parameter_nummer - 500, 2)

    if boom_parameter_nummer == 551:
        # Behandelklasse – som van (aantal * weegfactor 2)
        # Deze boomparameter wordt thans niet gebruikt door de grouper.
        # return BehandelKlasse(1)
        raise NotImplementedError('Boomparameter %d is niet geïmplementeerd.' % boom_parameter_nummer)

    raise RuntimeError("Onbekende boomparameter '%s'." % boom_parameter_nummer)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class ZorgActiviteitCluster(BoomParameter):
    """
    Klasse voor boomparameter zorgactiviteitcluster.

    Boomparameternummers: 301..310, 401..410, 401..510.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, cluster_nummer: int, weeg_factor_nummer: int):
        """
        Object constructor.

        :param int cluster_nummer: Het clusternummer (1..10):
        :param int weeg_factor_nummer: Weegfactornummer:

                                       * 0: geen weegfactor
                                       * 1: weegfactor 1
                                       * 2: weegfactor 2
        """
        self._cluster_nummer: int = cluster_nummer
        """
        Het clusternummer (1..10).
        """

        self._weeg_factor_nummer: int = weeg_factor_nummer
        """
        Weegfactornummer.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, cluster_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal zorgactiviteiten (met inachtneming van weegfactor) in een subtraject voorkomt in een
        zorgactiviteitcluster.

        :param str cluster_code: De zorgactiviteitclustercode.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_zorg_activiteit_cluster(cluster_code, self._cluster_nummer, self._weeg_factor_nummer)

# ----------------------------------------------------------------------------------------------------------------------



class Patient:
    """
    Klasse voor patiëntenn.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, geboorte_datum: str, geslacht_code: str):
        """
        Object constructor.

        :param str geboorte_datum: De geboortedatum van de patiënt.
        :param str geslacht_code: Het geslacht van de patiënt.
        """
        self.__geboorte_datum: str = geboorte_datum
        """
        De geboortedatum van de patiënt.
        """

        self.__geslacht_code: str = Patient.normaliseer_geslacht_code(geslacht_code)
        """
        Het geslacht van de patiënt.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @property
    def geslacht_code(self) -> str:
        """
        Geeft het geslacht van deze patiënt.

        :rtype: str
        """
        return self.__geslacht_code

    # ------------------------------------------------------------------------------------------------------------------
    def leeftijd(self, datum: str) -> int:
        """
        Geeft de leeftijd van deze patient op een peildatum.

        :param str datum: De peildatum.

        :rtype: int
        """
        if not datum:
            raise RuntimeError("Datum is niet gespecificeerd.")

        if not self.__geboorte_datum:
            raise RuntimeError("Geboortedatum is niet gespecificeerd.")

        if datum < self.__geboorte_datum:
            raise RuntimeError("Leeftijd van patient gevraagd op datum (%s) voor geboortedatum (%s)." %
                               (datum, self.__geboorte_datum))

        age = int(datum[0:4]) - int(self.__geboorte_datum[0:4])
        if datum[-5:] < self.__geboorte_datum[-5:]:
            age -= 1

        return age

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def normaliseer_geslacht_code(geslacht_code: str) -> str:
        """
        Normaliseert een geslachtscode naar 1 (man), 2 (vrouw) of 9 (anders).

        :param str geslacht_code: De geslachtscode.

        :rtype: str
        """
        if geslacht_code.upper() in ('1', 'M'):
            return '1'

        if geslacht_code.upper() in ('2', 'V', 'F'):
            return '2'

        return '9'

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.application.KerapuApplication import KerapuApplication


def main() -> None:
    """
    Entry point for the kerapu console script.
    """
    application = KerapuApplication()
    status = application.run()

    exit(status)



import csv
from typing import Optional, Dict, List, Tuple

from kerapu import clean_code, LEN_ZORG_ACTIVITEIT_CODE, clean_int, clean_str, clean_date, LEN_ZORG_PRODUCT_GROEP_CODE


class ZorgActiviteit:
    """
    Klasse voor zorgactiviteiten.
    """
    # ------------------------------------------------------------------------------------------------------------------
    __zorg_activiteiten_tabel: Dict[str, List[Dict[str, str]]] = {}
    """
    De zorgactiviteiten referentietabel.
    """

    __zorg_activiteiten_vertaal_tabel: Dict[str, List[Dict[str, str]]] = {}
    """
    De zorgactiviteiten vertaaltabel.
    """

    __behandel_klassen_tabel: Dict[Tuple[str, str, str], List[Dict[str, str]]] = {}
    """
    De behandelklassen referentietabel.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, zorg_activiteit_code: str, aantal: int):
        """
        Object constructor.

        :param str zorg_activiteit_code: De code van deze zorgactiviteit.
        :param int aantal: Het aantal malen dat deze zorgactiviteit is uitgevoerd.
        """
        self.__zorg_activiteit_code: str = clean_code(zorg_activiteit_code, LEN_ZORG_ACTIVITEIT_CODE)
        """
        De code van deze zorgactiviteit.
        """

        self.__aantal: int = clean_int(aantal, 0)
        """
        Het aantal malen dat deze zorgactiviteit is uitgevoerd.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __lees_zorgactiviteiten_tabel(folder: str) -> None:
        """
        Leest de zorgactiviteiten referentietabel (opgeslagen in CSV).

        :param str folder: De folder met alle goupertabellen.
        """
        with open(folder + '/ZorgActiviteiten.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                zorg_activiteit_code = clean_code(regel[0], LEN_ZORG_ACTIVITEIT_CODE)
                zorg_activiteit_cluster01 = clean_str(regel[2])
                zorg_activiteit_cluster02 = clean_str(regel[3])
                zorg_activiteit_cluster03 = clean_str(regel[4])
                zorg_activiteit_cluster04 = clean_str(regel[5])
                zorg_activiteit_cluster05 = clean_str(regel[6])
                zorg_activiteit_cluster06 = clean_str(regel[7])
                zorg_activiteit_cluster07 = clean_str(regel[8])
                zorg_activiteit_cluster08 = clean_str(regel[9])
                zorg_activiteit_cluster09 = clean_str(regel[10])
                zorg_activiteit_cluster10 = clean_str(regel[11])
                zorg_activiteit_weeg_factor1 = clean_int(regel[12])
                zorg_activiteit_weeg_factor2 = clean_int(regel[13])
                begin_datum = clean_date(regel[20])
                eind_datum = clean_date(regel[21])

                sleutel = zorg_activiteit_code

                rij = {'zorg_activiteit_code':         zorg_activiteit_code,
                       'zorg_activiteit_cluster01':    zorg_activiteit_cluster01,
                       'zorg_activiteit_cluster02':    zorg_activiteit_cluster02,
                       'zorg_activiteit_cluster03':    zorg_activiteit_cluster03,
                       'zorg_activiteit_cluster04':    zorg_activiteit_cluster04,
                       'zorg_activiteit_cluster05':    zorg_activiteit_cluster05,
                       'zorg_activiteit_cluster06':    zorg_activiteit_cluster06,
                       'zorg_activiteit_cluster07':    zorg_activiteit_cluster07,
                       'zorg_activiteit_cluster08':    zorg_activiteit_cluster08,
                       'zorg_activiteit_cluster09':    zorg_activiteit_cluster09,
                       'zorg_activiteit_cluster10':    zorg_activiteit_cluster10,
                       'zorg_activiteit_weeg_factor1': zorg_activiteit_weeg_factor1,
                       'zorg_activiteit_weeg_factor2': zorg_activiteit_weeg_factor2,
                       'begin_datum':                  begin_datum,
                       'eind_datum':                   eind_datum}

                if sleutel not in ZorgActiviteit.__zorg_activiteiten_tabel:
                    ZorgActiviteit.__zorg_activiteiten_tabel[sleutel] = []

                ZorgActiviteit.__zorg_activiteiten_tabel[sleutel].append(rij)

        print("Aantal zorgactiviteiten: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __lees_zorg_activiteiten_vertaal_tabel(folder: str) -> None:
        """
        Leest de zorgactiviteiten vertaaltabel (opgeslagen in CSV).

        :param str folder: De folder met alle goupertabellen.
        """
        with open(folder + '/VertaalZorgActiviteiten.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                zorg_activiteit_code = clean_code(regel[0], LEN_ZORG_ACTIVITEIT_CODE)
                zorg_activiteit_code_oud = clean_code(regel[2], LEN_ZORG_ACTIVITEIT_CODE)
                begin_datum = clean_date(regel[4])
                eind_datum = clean_date(regel[5])

                sleutel = zorg_activiteit_code

                rij = {'zorg_activiteit_code':     zorg_activiteit_code,
                       'zorg_activiteit_code_oud': zorg_activiteit_code_oud,
                       'begin_datum':              begin_datum,
                       'eind_datum':               eind_datum}

                if sleutel not in ZorgActiviteit.__zorg_activiteiten_vertaal_tabel:
                    ZorgActiviteit.__zorg_activiteiten_vertaal_tabel[sleutel] = []

                ZorgActiviteit.__zorg_activiteiten_vertaal_tabel[sleutel].append(rij)

        print("Aantal zorgactiviteit vertalingen: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __lees_behandel_klasse_tabel(folder: str) -> None:
        """
        Leest de behandelklasse tabel (opgeslagen in CSV).

        :param str folder: De folder met alle goupertabellen.
        """
        with open(folder + '/BehandelKlassen.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                zorg_product_groep_code = clean_code(regel[0], LEN_ZORG_PRODUCT_GROEP_CODE)
                zorg_activiteit_code = clean_code(regel[1], LEN_ZORG_ACTIVITEIT_CODE)
                behandel_klasse_code = clean_str(regel[2])
                begin_datum = clean_date(regel[4])
                eind_datum = clean_date(regel[5])

                sleutel = (zorg_product_groep_code, zorg_activiteit_code, behandel_klasse_code)

                rij = {'zorg_product_groep_code': zorg_product_groep_code,
                       'zorg_activiteit_code':    zorg_activiteit_code,
                       'behandel_klasse_code':    behandel_klasse_code,
                       'begin_datum':             begin_datum,
                       'eind_datum':              eind_datum}

                if sleutel not in ZorgActiviteit.__behandel_klassen_tabel:
                    ZorgActiviteit.__behandel_klassen_tabel[sleutel] = []

                ZorgActiviteit.__behandel_klassen_tabel[sleutel].append(rij)

        print("Aantal behandelklassen: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def init_static(folder: str) -> None:
        """
        Initialiseert alle statistische data.

        :param str folder: De folder met alle goupertabellen.
        """
        ZorgActiviteit.__lees_behandel_klasse_tabel(folder)
        ZorgActiviteit.__lees_zorgactiviteiten_tabel(folder)
        ZorgActiviteit.__lees_zorg_activiteiten_vertaal_tabel(folder)

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __vertaal_zorgactiviteit_code(zorg_activiteit_code: str, datum: str) -> str:
        """
        Vertaalt een "nieuwe" zorgactiviteitcode naar een "oude" zorgactiviteitcode die geldig is ten tijde van het
        begin van het subtraject.

        :param str zorg_activiteit_code: De "nieuwe" zorgactiviteitcode.
        :param str datum: De begindatum van het subtraject.
        """
        zorg_activiteit_code = clean_code(zorg_activiteit_code, LEN_ZORG_ACTIVITEIT_CODE)

        if zorg_activiteit_code in ZorgActiviteit.__zorg_activiteiten_vertaal_tabel:
            for vertaling in ZorgActiviteit.__zorg_activiteiten_vertaal_tabel[zorg_activiteit_code]:
                if vertaling['begin_datum'] <= datum <= vertaling['eind_datum']:
                    # Een geldige vertaling gevonden. geef de "oude" zorgactiviteitcode terug.
                    return vertaling['zorg_activiteit_code_oud']

            # Er is geen vertaling van toepassing ten tijde van het begin van het subtraject. Geef de gegeven
            # zorgactiviteitcode terug.
            return zorg_activiteit_code
        else:
            # Er is geen enkele vertaling voor de zorgactiviteitcode. Geef de gegeven zorgactiviteitcode terug.
            return zorg_activiteit_code

    # ------------------------------------------------------------------------------------------------------------------
    def __zorg_activiteit_referentie(self, datum: str) -> Optional[dict]:
        """
        Zoekt de referentie data voor deze zorgactiviteit in de zorgactiviteiten referentietabel.

        :param datum: De begindatum van het subtraject.

        :rtype: dict[str,int]
        """
        # Vertaal de zorgactiviteitcode naar een "oude" zorgactiviteitcode.
        zorg_activiteit_code = ZorgActiviteit.__vertaal_zorgactiviteit_code(self.__zorg_activiteit_code, datum)

        if zorg_activiteit_code in self.__zorg_activiteiten_tabel:
            for referentie in self.__zorg_activiteiten_tabel[zorg_activiteit_code]:
                if referentie['begin_datum'] <= datum <= referentie['eind_datum']:
                    # Een geldige referentie rij gevonden.
                    return referentie

            # Er is geen geldige referentie rij gevonden.
            return None

        else:
            return None

    # ------------------------------------------------------------------------------------------------------------------
    def __behandel_klasse_referentie(self,
                                     zorg_product_groep_code: str,
                                     behandel_klasse_code: str,
                                     datum: str) -> Optional[dict]:
        """
        Zoekt de referentie data voor deze zorgactiviteit in de behandelklasse referentietabel.

        :param str zorg_product_groep_code: De zorgproductgroepcode van het subtraject.
        :param str behandel_klasse_code: De gevraagde behandelklassecode
        :param str datum: De begindatum van het subtraject.

        :rtype: dict[str,int]
        """
        # Vertaal de zorgactiviteitcode naar een "oude" zorgactiviteitcode.
        zorg_activiteit_code = ZorgActiviteit.__vertaal_zorgactiviteit_code(self.__zorg_activiteit_code, datum)

        sleutel = (zorg_product_groep_code, zorg_activiteit_code, behandel_klasse_code)
        if sleutel in self.__behandel_klassen_tabel:
            for referentie in self.__behandel_klassen_tabel[sleutel]:
                if referentie['begin_datum'] <= datum <= referentie['eind_datum']:
                    # Een geldige referentie rij gevonden.
                    return referentie

        # Er is geen geldige referentie rij gevonden.
        return None

    # ------------------------------------------------------------------------------------------------------------------
    def behandel_klasse_aantal(self,
                               zorg_product_groep_code: str,
                               behandel_klasse_code: str,
                               weeg_factor_nummer: int,
                               datum: str) -> int:
        """
        Geeft het aantal malen (met inachtneming van weegfactor) dat deze zorgactiviteit voorkomt in een
        behandleklasse op een peildatum.

        :param str zorg_product_groep_code: De zorgproductgroepcode van het subtraject van deze zorgactiviteit.
        :param str behandel_klasse_code: De behandelklasse.
        :param int weeg_factor_nummer: Het weegfactornummer (0..2).
        :param str datum: De peildatum.

        :rtype: int
        """
        referentie = self.__behandel_klasse_referentie(zorg_product_groep_code, behandel_klasse_code, datum)

        if not referentie:
            # De (zorgproductgroepcode,zorgactiviteitcode,behandelklassecode) komt niet voor in de referentie tabel.
            # Geef 0 terug.
            return 0

        if weeg_factor_nummer == 0:
            # Weegfactor is niet van toepassing.
            return self.__aantal

        if weeg_factor_nummer == 1:
            return referentie['zorg_activiteit_weeg_factor1'] * self.__aantal

        if weeg_factor_nummer == 2:
            return referentie['zorg_activiteit_weeg_factor2'] * self.__aantal

        raise RuntimeError("Onbekend weegfactornummer %d." % weeg_factor_nummer)

    # ------------------------------------------------------------------------------------------------------------------
    def zorg_activiteit_aantal(self, zorg_activiteit_code: str, weeg_factor_nummer: int, datum: str) -> int:
        """
        Geeft het aantal malen (met inachtneming van weegfactor) dat deze zorgactiviteit voldoet aan een
        zorgactiviteitcode.

        :param str zorg_activiteit_code: De zorgactiviteitcode.
        :param int weeg_factor_nummer: Het weegfactornummer (0..2).
        :param str datum: De peildatum.

        :rtype: int
        """
        referentie = self.__zorg_activiteit_referentie(datum)

        if not referentie:
            # De zorgactiviteitcode komt niet voor in de referentie tabel. Geef 0 terug.
            return 0

        if self.__zorg_activiteit_code != clean_code(zorg_activiteit_code, LEN_ZORG_ACTIVITEIT_CODE):
            return 0

        if weeg_factor_nummer == 0:
            # Weegfactor is niet van toepassing.
            return self.__aantal

        if weeg_factor_nummer == 1:
            return referentie['zorg_activiteit_weeg_factor1'] * self.__aantal

        if weeg_factor_nummer == 2:
            return referentie['zorg_activiteit_weeg_factor2'] * self.__aantal

        raise RuntimeError("Onbekend weegfactornummer %d." % weeg_factor_nummer)

    # ------------------------------------------------------------------------------------------------------------------
    def zorg_activiteit_cluster_aantal(self,
                                       cluster_code: str,
                                       cluster_nummer: int,
                                       weeg_factor_nummer: int,
                                       datum: str) -> int:
        """
        Geeft het aantal malen (met inachtneming van weegfactor) dat deze zorgactiviteit voorkomt in een
        zorgactiviteitcluster.

        :param str cluster_code: De zorgactiviteitclustercode.
        :param int cluster_nummer: het cluster nummber (1..10).
        :param int weeg_factor_nummer: Het weegfactornummer (0..2).
        :param str datum: De peildatum.

        :rtype: int
        """
        referentie = self.__zorg_activiteit_referentie(datum)

        if not referentie:
            # De zorgactiviteitcode komt niet voor in de referentie tabel. Geef 0 terug.
            return 0

        if 1 <= cluster_nummer <= 10:
            if referentie['zorg_activiteit_cluster%02d' % cluster_nummer] != cluster_code:
                # Deze zorgactiviteit kom niet voor in het gevraagde cluster.
                return 0

            if weeg_factor_nummer == 0:
                # Weegfactor is niet van toepassing.
                return self.__aantal

            if weeg_factor_nummer == 1:
                return referentie['zorg_activiteit_weeg_factor1'] * self.__aantal

            if weeg_factor_nummer == 2:
                return referentie['zorg_activiteit_weeg_factor2'] * self.__aantal

            raise RuntimeError("Onbekend weegfactornummer %d." % weeg_factor_nummer)

        raise RuntimeError("Onbekend clusternummer %d." % cluster_nummer)

# ----------------------------------------------------------------------------------------------------------------------



import csv
from typing import Optional, Dict, List

from kerapu import clean_code, clean_date, clean_str, LEN_SPECIALISME_CODE


class Specialisme:
    """
    Klasse voor specialismen.
    """
    # ------------------------------------------------------------------------------------------------------------------
    _specialisme_tabel: Dict[str, List[Dict[str, str]]] = {}
    """
    De specialismen referentietabel.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, specialisme_code: str) -> None:
        """
        Object constructor.

        :param str specialisme_code: De code van het specialisme.
        """
        self.__specialisme_code: str = clean_code(specialisme_code, LEN_SPECIALISME_CODE)
        """
        De code van het uitvoerend specialisme.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def __lees_specialisme_tabel(folder: str) -> None:
        """
        Leest de specialisme referentietabel (opgeslagen in CSV).

        :param str folder: De folder met alle goupertabellen.
        """
        with open(folder + '/Specialismen.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                specialisme_code = clean_code(regel[0], LEN_SPECIALISME_CODE)
                specialisme_cluster1 = clean_str(regel[4])
                specialisme_cluster2 = clean_str(regel[5])
                begin_datum = clean_date(regel[7])
                eind_datum = clean_date(regel[8])

                rij = {'specialisme_code':     specialisme_code,
                       'specialisme_cluster1': specialisme_cluster1,
                       'specialisme_cluster2': specialisme_cluster2,
                       'begin_datum':          begin_datum,
                       'eind_datum':           eind_datum}

                if specialisme_code not in Specialisme._specialisme_tabel:
                    Specialisme._specialisme_tabel[specialisme_code] = []

                Specialisme._specialisme_tabel[specialisme_code].append(rij)

        print("Aantal specialismen: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    @staticmethod
    def init_static(folder: str) -> None:
        """
        Initialiseert alle statistische data.

        :param str folder: De folder met alle goupertabellen.
        """
        Specialisme.__lees_specialisme_tabel(folder)

    # ------------------------------------------------------------------------------------------------------------------
    def __get_specialisme_referentie(self, datum: str) -> Optional[Dict]:
        """
        Zoekt de referentie data voor deze specialisme in de specialismen referentietabel.

        :param str datum: De begindatum van het subtraject.

        :rtype: dict[str,str]
        """
        if self.__specialisme_code in self._specialisme_tabel:
            for referentie in self._specialisme_tabel[self.__specialisme_code]:
                if referentie['begin_datum'] <= datum <= referentie['eind_datum']:
                    # Een geldige referentie rij gevonden.
                    return referentie

            # Er is geen geldige referentie rij gevonden.
            return None

        else:
            return None

    # ------------------------------------------------------------------------------------------------------------------
    def specialisme_aantal(self, specialisme_code: str, datum: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat dit specialisme voldoet aan een attributecode op een gegeven datum
        .
        :param str specialisme_code: De attribuutcode waaraan voldaan moet worden.
        :param str datum: De datum.

        :rtype: int
        """
        referentie = self.__get_specialisme_referentie(datum)

        if not referentie:
            # Het specialisme komt niet voor in de referentie tabel. Geef 0 terug.
            return 0

        if referentie['specialisme_code'] == clean_code(specialisme_code, LEN_SPECIALISME_CODE):
            return 1

        return 0

    # ------------------------------------------------------------------------------------------------------------------
    def specialisme_cluster_aantal(self, cluster_code: str, cluster_nummer: int, datum: str) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat dit specialisme voldoet aan een clustercode op een gegeven datum.

        :param str cluster_code: De clustercode waaraan voldaan moet worden.
        :param int cluster_nummer: Het clusternummer.
        :param str datum: De datum.

        :rtype: int
        """
        referentie = self.__get_specialisme_referentie(datum)

        if not referentie:
            # Het specialisme komt niet voor in de referentie tabel. Geef 0 terug.
            return 0

        if cluster_nummer == 0:
            return 1

        if 1 <= cluster_nummer <= 2:
            if referentie['specialisme_cluster%d' % cluster_nummer] == cluster_code:
                # Dit specialisme komt voor in het gevraagde cluster.
                return 1

            return 0

        raise RuntimeError("Onbekend clusternummer %d." % cluster_nummer)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class Geslacht(BoomParameter):
    """
    Klasse voor boomparameter geslacht.

    Boomparameternummer: 230.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, geslacht_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat de patient van een subtraject voldoet aan een geslacht.

        :param str geslacht_code: De geslachtscode waartegen getest moet worden.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_geslacht_code(geslacht_code)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class ZorgInstellingCode(BoomParameter):
    """
    Klasse voor boomparameter zorginstelling.

    Boomparameternummer: 110.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, zorg_instelling_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat het zorginstelling van een subtraject voldoet een zorginstellingcode.

        :param str zorg_instelling_code: De AGB-code waaraan de zorginstelling moet voldoen.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_zorg_instelling(zorg_instelling_code)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class ZorgTypeCode(BoomParameter):
    """
    Klasse voor boomparameter zorgtype.

    Boomparameternummer: 210.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, zorg_type_attribuut_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat het zorgtype van een subtraject voldoet aan een
        (specialismecode, zorgtypecode) combinatie.

        :param str zorg_type_attribuut_code: De attribuutcode voor (specialismecode, zorgtypecode) combinatie.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_zorg_type_attribuut(zorg_type_attribuut_code)

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class SpecialismeCode(BoomParameter):
    """
    Klasse voor boomparameter specialismecode.

    Boomparameternummer: 200.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, specialisme_code: str, subtraject: Subtraject) -> int:
        """
        Geeft het aantal malen (d.w.z. 0 of 1) dat het uitvoerend specialisme van een subtraject voldoet aan een
        specialismecode.

        :param str specialisme_code: De specialismecode.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.telling_specialisme(specialisme_code)

# ----------------------------------------------------------------------------------------------------------------------



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as handle:
    long_description = handle.read()

setup(
    name='Kerapu',

    version='2.0.3',

    description='Een implementatie van de grouper',
    long_description=long_description,

    url='https://github.com/SetBased/py-kerapu',

    author='Paul Water',
    author_email='p.r.water@setbased.nl',

    license='MIT',

    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',

        'License :: OSI Approved :: MIT License',
        'Natural Language :: Dutch',
        'Operating System :: OS Independent',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    keywords='DBC Grouper',

    packages=find_packages(exclude=['build', 'test']),

    entry_points={
       'console_scripts': [
           'kerapu = kerapu.application.kerapu:main',
       ],
    },

    install_requires=['lxml', 'cleo==0.6.8']
)



from typing import List

from kerapu.boom.attribuut_groep_koppeling import AttribuutGroepKoppeling
from kerapu.lbz.Subtraject import Subtraject


class AttribuutGroep:
    """
    Klasse voor attribuutgroep.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self,
                 attribute_groep_id: int,
                 aantal_voorwaarden_voor_true: int,
                 koppelingen: List[AttribuutGroepKoppeling]):
        """
        Object constructor.

        :param int attribute_groep_id: Het ID van deze attribuutgroep.
        :param int aantal_voorwaarden_voor_true: Het minimale aantal voorwaarden waaraan moet worden voldaan.
        :param list[kerapu.boom.attribuut_groep_koppeling.AttribuutGroepKoppeling.AttribuutGroepKoppeling] koppelingen:
               De attribuutgroepkoppelingen.
        """
        self._attribute_groep_id: int = attribute_groep_id
        """
        Het ID van deze attribuutgroep.
        """

        self._aantal_voorwaarden_voor_true: int = aantal_voorwaarden_voor_true
        """
        Het minimale aantal voorwaarden waaraan moet worden voldaan om deze attribuutgroep te laten vuren.
        """

        self._koppelingen: List[AttribuutGroepKoppeling] = koppelingen
        """
        De attribuutgroepkoppelingen.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def test(self, subtraject: Subtraject) -> bool:
        """
        Test of een subtraject voldoet aan de voorwaarden van deze attribuutgroep.

        :param Subtraject subtraject: Het subtraject.

        :rtype: bool
        """
        aantal = 0
        for koppeling in self._koppelingen:
            if koppeling.test(subtraject):
                aantal += 1

        return aantal >= self._aantal_voorwaarden_voor_true

# ----------------------------------------------------------------------------------------------------------------------



from kerapu.boom.boom_parameter.BoomParameter import BoomParameter
from kerapu.lbz.Subtraject import Subtraject


class Leeftijd(BoomParameter):
    """
    Klasse voor boomparameter leeftijd.

    Boomparameternummer: 100.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def tel(self, leeg, subtraject: Subtraject) -> int:
        """
        Geeft de leeftijd van de patient van een subtraject.

        :param None leeg: Wordt niet gebruikt.
        :param Subtraject subtraject: Het subtraject.

        :rtype: int
        """
        return subtraject.leeftijd

# ----------------------------------------------------------------------------------------------------------------------



from typing import Optional

from kerapu.boom.AttribuutGroep import AttribuutGroep
from kerapu.lbz.Subtraject import Subtraject


class BeslisRegel:
    """
    Klasse voor beslisregels.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self,
                 beslist_regel_id: int,
                 attribuut_groep: AttribuutGroep,
                 label_true: str,
                 label_false: str,
                 indicatie_aanspraakbeperking: bool):
        """
        Object constructor.

        :param int beslist_regel_id: Het ID van deze beslisregel.
        :param AttribuutGroep attribuut_groep: De attribuutgroep van deze beslisregel.
        :param str label_true: Label voor True.
        :param str label_false: Label voor False.
        :param bool indicatie_aanspraakbeperking: Vlag voor aanspraakbeperking.
        """
        self._beslist_regel_id: int = beslist_regel_id
        """
         Het ID van deze beslisregel.
        """

        self._attribuut_groep: AttribuutGroep = attribuut_groep
        """
        De attribuutgroep van deze beslisregel.
        """

        self._beslist_regel_true: Optional[BeslisRegel] = None
        """
        De beslisregel die gevolgt moet worden als deze beslisregel True is.
        """

        self._beslist_regel_false: Optional[BeslisRegel] = None
        """
        De beslisregel die gevolgt moet worden als deze beslisregel False is.
        """

        self._label_true: str = label_true
        """
        Label voor True.
        """

        self._label_false: str = label_false
        """
        Label voor False.
        """

        self._indicatie_aanspraakbeperking: bool = indicatie_aanspraakbeperking
        """
        Vlag voor aanspraakbeperking.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def verrijk(self, beslist_regel_true, beslist_regel_false) -> None:
        """
        Verrijkt deze beslisregel met beslisregels voor True and False.

        :param kerapu.boom.BeslisRegel.BeslisRegel beslist_regel_true: De beslisregel voor True.
        :param kerapu.boom.BeslisRegel.BeslisRegel beslist_regel_false: De beslisregel voor False.
        """
        self._beslist_regel_true = beslist_regel_true
        self._beslist_regel_false = beslist_regel_false

    # ------------------------------------------------------------------------------------------------------------------
    def klim(self, subtraject: Subtraject) -> str:
        """
        Klimt door de beslisboom een geeft het uiteindelijk gevonden label terug.

        :param Subtraject subtraject: Het subtraject.

        :rtype: str
        """
        test = self._attribuut_groep.test(subtraject)

        if not test:
            if self._beslist_regel_false:
                return self._beslist_regel_false.klim(subtraject)

            return self._label_false

        if self._beslist_regel_true:
            return self._beslist_regel_true.klim(subtraject)

        return self._label_true

# ----------------------------------------------------------------------------------------------------------------------



import csv
from typing import Dict, List

from kerapu import clean_str, clean_int, clean_bool, clean_date
from kerapu.boom.attribuut import maak_attribuut, Attribuut
from kerapu.boom.AttribuutGroep import AttribuutGroep
from kerapu.boom.attribuut_groep_koppeling import maak_attribuut_groep_koppeling, AttribuutGroepKoppeling
from kerapu.boom.BeslisRegel import BeslisRegel
from kerapu.boom.ZorgProductGroep import ZorgProductGroep
from kerapu.boom.ZorgProductGroepVersie import ZorgProductGroepVersie
from kerapu.lbz.Diagnose import Diagnose
from kerapu.lbz.Specialisme import Specialisme
from kerapu.lbz.Subtraject import Subtraject
from kerapu.lbz.ZorgActiviteit import ZorgActiviteit
from kerapu.lbz.ZorgType import ZorgType
from kerapu.lbz.ZorgVraag import ZorgVraag


class Kerapu:
    """
    Een implementatie van de grouper in Python.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self):
        """
        Object constructor.
        """
        self.__zorgproductgroep_boom: Dict[str, ZorgProductGroep] = {}
        """
        De zorgproductgroepboom. Sleutel is zorgproductgroepcode.
        """

        self.__beslisregels: Dict[int, BeslisRegel] = {}
        """
        Alle beslisregels. Sleutel is het ID van de beslisregel.
        """

        self.__attribuutgroepen: Dict[int, AttribuutGroep] = {}
        """
        Alle attribuutgroepen. Sleutel is het ID van de attribuutgroep.
        """

        self.__attribuut_groep_koppelingen: Dict[int, List[AttribuutGroepKoppeling]] = {}
        """
        Alle attribuutgroepkoppelingen. Sleutel is het ID van de attribuutgroep.
        """

        self.__attributen: Dict[int, Attribuut] = {}
        """
        Alle attributen. Sleutel is het ID van het attribuut.
        """

    # ------------------------------------------------------------------------------------------------------------------
    def init_static(self, folder: str) -> None:
        """
        Initialiseert alle statistische data.

        :param str folder: De folder met alle goupertabellen.
        """
        self.__lees_attribuut_tabel(folder)
        self.__lees_attribuut_groep_koppeling_tabel(folder)
        self.__lees_attribuut_groepen_tabel(folder)
        self.__lees_beslis_regel_tabel(folder)
        self.__lees_zorg_product_groepen(folder)

        Diagnose.init_static(folder)
        Specialisme.init_static(folder)
        ZorgActiviteit.init_static(folder)
        ZorgType.init_static(folder)
        ZorgVraag.init_static(folder)

    # ------------------------------------------------------------------------------------------------------------------
    def __lees_attribuut_tabel(self, folder: str) -> None:
        """
        Leest de attribuuttabel (opgeslagen in CSV).

        :param str folder: De folder met alle groupertabellen in CSV-formaat.
        """
        with open(folder + '/Attributen.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                attribuut_id = int(regel[0])
                boom_parameter_nummer = int(regel[2])
                filter_toets_wijze = int(regel[3])
                filter_waarde_type = int(regel[4])
                onder_filter_waarde = clean_str(regel[5])
                boven_filter_waarde = clean_str(regel[6])

                self.__attributen[attribuut_id] = maak_attribuut(attribuut_id,
                                                                 boom_parameter_nummer,
                                                                 filter_toets_wijze,
                                                                 filter_waarde_type,
                                                                 onder_filter_waarde,
                                                                 boven_filter_waarde)

        print("Aantal attributen: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    def __lees_attribuut_groep_koppeling_tabel(self, folder: str) -> None:
        """
        Leest de  attribuutgroepkoppelingen (opgeslagen in CSV).

        :param str folder: De folder met alle groupertabellen in CSV-formaat.
        """
        with open(folder + '/AttribuutGroepKoppelingen.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                attribuut_groep_id = int(regel[1])
                attribuut_id = int(regel[2])
                attribuut_toets_wijze = int(regel[3])
                onder_toets_waarde = clean_int(regel[4])
                boven_toets_waarde = clean_int(regel[5])

                if attribuut_groep_id not in self.__attribuut_groep_koppelingen:
                    self.__attribuut_groep_koppelingen[attribuut_groep_id] = []

                if attribuut_id not in self.__attributen:
                    raise RuntimeError("Onbekend attribuut: '%d'" % attribuut_id)

                koppeling = maak_attribuut_groep_koppeling(attribuut_groep_id,
                                                           self.__attributen[attribuut_id],
                                                           attribuut_toets_wijze,
                                                           onder_toets_waarde,
                                                           boven_toets_waarde)

                self.__attribuut_groep_koppelingen[attribuut_groep_id].append(koppeling)

        print("Aantal attribuutgroepkoppelingen: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    def __lees_attribuut_groepen_tabel(self, folder: str) -> None:
        """
        Leest de attribuutgroepen (opgeslagen in CSV).

        :param str folder: De folder met alle groupertabellen in CSV-formaat.
        """
        with open(folder + '/AttribuutGroepen.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                attribuut_groep_id = int(regel[0])
                aantal_voorwaarden_voor_true = int(regel[2])

                if attribuut_groep_id not in self.__attribuut_groep_koppelingen:
                    raise RuntimeError("Onbekende koppeling: '%d'" % attribuut_groep_id)

                self.__attribuutgroepen[attribuut_groep_id] = AttribuutGroep(attribuut_groep_id,
                                                                             aantal_voorwaarden_voor_true,
                                                                             self.__attribuut_groep_koppelingen[
                                                                                 attribuut_groep_id])

        print("Aantal attributen: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    def __lees_beslis_regel_tabel(self, folder: str) -> None:
        """
        Leest de beslisregels (opgeslagen in CSV).

        :param str folder: De folder met alle groupertabellen in CSV-formaat.
        """
        verrijkingen = {}
        with open(folder + '/BeslisRegels.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                beslis_regel_id = int(regel[0])
                attribuut_groep_id = int(regel[1])
                beslist_regel_id_true = clean_int(regel[2])
                beslist_regel_id_false = clean_int(regel[3])
                label_true = clean_str(regel[4])
                label_false = clean_str(regel[5])
                indicatie_aanspraakbeperking = clean_bool(regel[6])

                if attribuut_groep_id not in self.__attribuutgroepen:
                    raise RuntimeError("Onbekende attribuutgroep: '%d'" % attribuut_groep_id)

                verrijkingen[beslis_regel_id] = (beslist_regel_id_true, beslist_regel_id_false)

                self.__beslisregels[beslis_regel_id] = BeslisRegel(beslis_regel_id,
                                                                   self.__attribuutgroepen[attribuut_groep_id],
                                                                   label_true,
                                                                   label_false,
                                                                   indicatie_aanspraakbeperking)

        # Verrijk alle beslisregels met beslisregels voor true en false.
        for beslis_regel_id, (beslist_regel_id_true, beslist_regel_id_false) in verrijkingen.items():
            beslist_regel_true = self.__beslisregels.get(beslist_regel_id_true, None)
            beslist_regel_false = self.__beslisregels.get(beslist_regel_id_false, None)
            self.__beslisregels[beslis_regel_id].verrijk(beslist_regel_true, beslist_regel_false)

        print("Aantal beslisregels: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    def __lees_zorg_product_groepen(self, folder: str) -> None:
        """
        Leest de zorgproductgroepen (opgeslagen in CSV).

        :param str folder: De folder met alle groupertabellen in CSV-formaat.
        """
        with open(folder + '/ZorgProductGroepen.csv', 'r', encoding='utf-8') as csv_file:
            reader = csv.reader(csv_file)
            regel_nummer = 0
            for regel in reader:
                regel_nummer += 1

                # Sla de eerste regel met koppen over.
                if regel_nummer == 1:
                    continue

                zorg_product_groep_code = clean_str(regel[0])
                beslist_regel_id_start = int(regel[2])
                begin_datum = clean_date(regel[3])
                eind_datum = clean_date(regel[4])

                if zorg_product_groep_code not in self.__zorgproductgroep_boom:
                    self.__zorgproductgroep_boom[zorg_product_groep_code] = ZorgProductGroep(zorg_product_groep_code)

                versie = ZorgProductGroepVersie(zorg_product_groep_code,
                                                self.__beslisregels[beslist_regel_id_start],
                                                begin_datum,
                                                eind_datum)
                self.__zorgproductgroep_boom[zorg_product_groep_code].versie_toevoegen(versie)

        print("Aantal zorgproductgroep versies: %d" % (regel_nummer - 1))

    # ------------------------------------------------------------------------------------------------------------------
    def bepaal_zorg_product_groep(self, subtraject: Subtraject) -> str:
        """
        Bepaalt de zorgproductgroep van een subtraject.

        :param Subtraject subtraject: Het subtraject waarvoor de zorgproductgroep moet worden bepaalt.

        :rtype: str
        """
        top_boom = self.__zorgproductgroep_boom['0']
        subtraject.zorg_product_groep_code = top_boom.klim(subtraject)

        return subtraject.zorg_product_groep_code

    # ------------------------------------------------------------------------------------------------------------------
    def bepaal_zorg_product(self, subtraject: Subtraject) -> str:
        """
        Bepaalt de zorgproduct van een subtraject.

        :param Subtraject subtraject: Het subtraject waarvoor de zorgproductcode moet worden bepaalt.

        :rtype: str
        """
        zorg_product_groep_code = self.bepaal_zorg_product_groep(subtraject)

        if zorg_product_groep_code is not None:
            top_boom = self.__zorgproductgroep_boom[zorg_product_groep_code]
            subtraject.zorg_product_code = top_boom.klim(subtraject)
        else:
            subtraject.zorg_product_code = None

        return subtraject.zorg_product_code

# ----------------------------------------------------------------------------------------------------------------------



import abc

from kerapu.boom.attribuut.Attribuut import Attribuut
from kerapu.lbz.Subtraject import Subtraject


class AttribuutGroepKoppeling:
    """
    Abstract klasse voor attribuutgroepkoppelingen.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, attribute_groep_id: int, attribuut: Attribuut):
        """
        Object constructor.

        :param int attribute_groep_id: Het ID van deze koppeling.
        :param Attribuut attribuut: Het attribuut van deze koppeling.
        """
        self._attribute_groep_id: int = attribute_groep_id
        """
        Het ID van deze attribuutgroepkoppeling.
        """

        self._attribuut: Attribuut = attribuut
        """
        Het attribuut van deze koppeling.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @abc.abstractmethod
    def test(self, subtraject: Subtraject) -> bool:
        """
        Test of een subtraject voldoet aan een attribuutgroepkoppeling.

        :param Subtraject subtraject: Het subtraject.

        :rtype: bool
        """""
        pass

# ----------------------------------------------------------------------------------------------------------------------
